@@ -11,7 +11,6 @@ import (
 	"context"
 	"flag"
 	"fmt"
-	"go/token"
 	"io/ioutil"
 	"log"
 	"os"
@@ -23,8 +22,10 @@ import (
 	"time"
 
 	"golang.org/x/tools/gopls/internal/lsp"
+	"golang.org/x/tools/gopls/internal/lsp/browser"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/debug"
+	"golang.org/x/tools/gopls/internal/lsp/filecache"
 	"golang.org/x/tools/gopls/internal/lsp/lsprpc"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
@@ -215,6 +216,11 @@ func isZeroValue(f *flag.Flag, value string) bool {
 // If no arguments are passed it will invoke the server sub command, as a
 // temporary measure for compatibility.
 func (app *Application) Run(ctx context.Context, args ...string) error {
+	// In the category of "things we can do while waiting for the Go command":
+	// Pre-initialize the filecache, which takes ~50ms to hash the gopls
+	// executable, and immediately runs a gc.
+	filecache.Start()
+
 	ctx = debug.WithInstance(ctx, app.wd, app.OCAgent)
 	if len(args) == 0 {
 		s := flag.NewFlagSet(app.Name(), flag.ExitOnError)
@@ -230,7 +236,7 @@ func (app *Application) Run(ctx context.Context, args ...string) error {
 	return tool.CommandLineErrorf("Unknown command %v", command)
 }
 
-// commands returns the set of commands supported by the gopls tool on the
+// Commands returns the set of commands supported by the gopls tool on the
 // command line.
 // The command is specified by the first non flag argument.
 func (app *Application) Commands() []tool.Application {
@@ -269,9 +275,9 @@ func (app *Application) featureCommands() []tool.Application {
 		&rename{app: app},
 		&semtok{app: app},
 		&signature{app: app},
+		&stats{app: app},
 		&suggestedFix{app: app},
 		&symbols{app: app},
-		newWorkspace(app),
 		&workspaceSymbol{app: app},
 		&vulncheck{app: app},
 	}
@@ -282,13 +288,20 @@ var (
 	internalConnections = make(map[string]*connection)
 )
 
-func (app *Application) connect(ctx context.Context) (*connection, error) {
+// connect creates and initializes a new in-process gopls session.
+//
+// If onProgress is set, it is called for each new progress notification.
+func (app *Application) connect(ctx context.Context, onProgress func(*protocol.ProgressParams)) (*connection, error) {
 	switch {
 	case app.Remote == "":
-		connection := newConnection(app)
-		connection.Server = lsp.NewServer(cache.NewSession(ctx, cache.New(nil, nil), app.options), connection.Client)
-		ctx = protocol.WithClient(ctx, connection.Client)
-		return connection, connection.initialize(ctx, app.options)
+		client := newClient(app, onProgress)
+		server := lsp.NewServer(cache.NewSession(ctx, cache.New(nil), app.options), client)
+		conn := newConnection(server, client)
+		if err := conn.initialize(protocol.WithClient(ctx, client), app.options); err != nil {
+			return nil, err
+		}
+		return conn, nil
+
 	case strings.HasPrefix(app.Remote, "internal@"):
 		internalMu.Lock()
 		defer internalMu.Unlock()
@@ -323,19 +336,19 @@ func CloseTestConnections(ctx context.Context) {
 }
 
 func (app *Application) connectRemote(ctx context.Context, remote string) (*connection, error) {
-	connection := newConnection(app)
 	conn, err := lsprpc.ConnectToRemote(ctx, remote)
 	if err != nil {
 		return nil, err
 	}
 	stream := jsonrpc2.NewHeaderStream(conn)
 	cc := jsonrpc2.NewConn(stream)
-	connection.Server = protocol.ServerDispatcher(cc)
-	ctx = protocol.WithClient(ctx, connection.Client)
+	server := protocol.ServerDispatcher(cc)
+	client := newClient(app, nil)
+	connection := newConnection(server, client)
+	ctx = protocol.WithClient(ctx, connection.client)
 	cc.Go(ctx,
 		protocol.Handlers(
-			protocol.ClientHandler(connection.Client,
-				jsonrpc2.MethodNotFound)))
+			protocol.ClientHandler(client, jsonrpc2.MethodNotFound)))
 	return connection, connection.initialize(ctx, app.options)
 }
 
@@ -347,7 +360,7 @@ var matcherString = map[source.SymbolMatcher]string{
 
 func (c *connection) initialize(ctx context.Context, options func(*source.Options)) error {
 	params := &protocol.ParamInitialize{}
-	params.RootURI = protocol.URIFromPath(c.Client.app.wd)
+	params.RootURI = protocol.URIFromPath(c.client.app.wd)
 	params.Capabilities.Workspace.Configuration = true
 
 	// Make sure to respect configured options when sending initialize request.
@@ -356,16 +369,23 @@ func (c *connection) initialize(ctx context.Context, options func(*source.Option
 		options(opts)
 	}
 	// If you add an additional option here, you must update the map key in connect.
-	params.Capabilities.TextDocument.Hover = protocol.HoverClientCapabilities{
+	params.Capabilities.TextDocument.Hover = &protocol.HoverClientCapabilities{
 		ContentFormat: []protocol.MarkupKind{opts.PreferredContentFormat},
 	}
 	params.Capabilities.TextDocument.DocumentSymbol.HierarchicalDocumentSymbolSupport = opts.HierarchicalDocumentSymbolSupport
 	params.Capabilities.TextDocument.SemanticTokens = protocol.SemanticTokensClientCapabilities{}
-	params.Capabilities.TextDocument.SemanticTokens.Formats = []string{"relative"}
-	params.Capabilities.TextDocument.SemanticTokens.Requests.Range = true
-	params.Capabilities.TextDocument.SemanticTokens.Requests.Full = true
+	params.Capabilities.TextDocument.SemanticTokens.Formats = []protocol.TokenFormat{"relative"}
+	params.Capabilities.TextDocument.SemanticTokens.Requests.Range.Value = true
+	params.Capabilities.TextDocument.SemanticTokens.Requests.Full.Value = true
 	params.Capabilities.TextDocument.SemanticTokens.TokenTypes = lsp.SemanticTypes()
 	params.Capabilities.TextDocument.SemanticTokens.TokenModifiers = lsp.SemanticModifiers()
+
+	// If the subcommand has registered a progress handler, report the progress
+	// capability.
+	if c.client.onProgress != nil {
+		params.Capabilities.Window.WorkDoneProgress = true
+	}
+
 	params.InitializationOptions = map[string]interface{}{
 		"symbolMatcher": matcherString[opts.SymbolMatcher],
 	}
@@ -380,36 +400,40 @@ func (c *connection) initialize(ctx context.Context, options func(*source.Option
 
 type connection struct {
 	protocol.Server
-	Client *cmdClient
+	client *cmdClient
 }
 
+// cmdClient defines the protocol.Client interface behavior of the gopls CLI tool.
 type cmdClient struct {
-	protocol.Server
-	app  *Application
-	fset *token.FileSet
+	app        *Application
+	onProgress func(*protocol.ProgressParams)
 
 	diagnosticsMu   sync.Mutex
 	diagnosticsDone chan struct{}
 
-	filesMu sync.Mutex
+	filesMu sync.Mutex // guards files map and each cmdFile.diagnostics
 	files   map[span.URI]*cmdFile
 }
 
 type cmdFile struct {
 	uri         span.URI
-	mapper      *protocol.ColumnMapper
+	mapper      *protocol.Mapper
 	err         error
-	open        bool
 	diagnostics []protocol.Diagnostic
 }
 
-func newConnection(app *Application) *connection {
+func newClient(app *Application, onProgress func(*protocol.ProgressParams)) *cmdClient {
+	return &cmdClient{
+		app:        app,
+		onProgress: onProgress,
+		files:      make(map[span.URI]*cmdFile),
+	}
+}
+
+func newConnection(server protocol.Server, client *cmdClient) *connection {
 	return &connection{
-		Client: &cmdClient{
-			app:   app,
-			fset:  token.NewFileSet(),
-			files: make(map[span.URI]*cmdFile),
-		},
+		Server: server,
+		client: client,
 	}
 }
 
@@ -516,22 +540,73 @@ func (c *cmdClient) PublishDiagnostics(ctx context.Context, p *protocol.PublishD
 	defer c.filesMu.Unlock()
 
 	file := c.getFile(ctx, fileURI(p.URI))
-	file.diagnostics = p.Diagnostics
+	file.diagnostics = append(file.diagnostics, p.Diagnostics...)
+
+	// Perform a crude in-place deduplication.
+	// TODO(golang/go#60122): replace the ad-hoc gopls/diagnoseFiles
+	// non-standard request with support for textDocument/diagnostic,
+	// so that we don't need to do this de-duplication.
+	type key [6]interface{}
+	seen := make(map[key]bool)
+	out := file.diagnostics[:0]
+	for _, d := range file.diagnostics {
+		var codeHref string
+		if desc := d.CodeDescription; desc != nil {
+			codeHref = desc.Href
+		}
+		k := key{d.Range, d.Severity, d.Code, codeHref, d.Source, d.Message}
+		if !seen[k] {
+			seen[k] = true
+			out = append(out, d)
+		}
+	}
+	file.diagnostics = out
+
 	return nil
 }
 
-func (c *cmdClient) Progress(context.Context, *protocol.ProgressParams) error {
+func (c *cmdClient) Progress(_ context.Context, params *protocol.ProgressParams) error {
+	if c.onProgress != nil {
+		c.onProgress(params)
+	}
 	return nil
 }
 
-func (c *cmdClient) ShowDocument(context.Context, *protocol.ShowDocumentParams) (*protocol.ShowDocumentResult, error) {
-	return nil, nil
+func (c *cmdClient) ShowDocument(ctx context.Context, params *protocol.ShowDocumentParams) (*protocol.ShowDocumentResult, error) {
+	var success bool
+	if params.External {
+		// Open URI in external browser.
+		success = browser.Open(string(params.URI))
+	} else {
+		// Open file in editor, optionally taking focus and selecting a range.
+		// (cmdClient has no editor. Should it fork+exec $EDITOR?)
+		log.Printf("Server requested that client editor open %q (takeFocus=%t, selection=%+v)",
+			params.URI, params.TakeFocus, params.Selection)
+		success = true
+	}
+	return &protocol.ShowDocumentResult{Success: success}, nil
 }
 
 func (c *cmdClient) WorkDoneProgressCreate(context.Context, *protocol.WorkDoneProgressCreateParams) error {
 	return nil
 }
 
+func (c *cmdClient) DiagnosticRefresh(context.Context) error {
+	return nil
+}
+
+func (c *cmdClient) InlayHintRefresh(context.Context) error {
+	return nil
+}
+
+func (c *cmdClient) SemanticTokensRefresh(context.Context) error {
+	return nil
+}
+
+func (c *cmdClient) InlineValueRefresh(context.Context) error {
+	return nil
+}
+
 func (c *cmdClient) getFile(ctx context.Context, uri span.URI) *cmdFile {
 	file, found := c.files[uri]
 	if !found || file.err != nil {
@@ -541,19 +616,12 @@ func (c *cmdClient) getFile(ctx context.Context, uri span.URI) *cmdFile {
 		c.files[uri] = file
 	}
 	if file.mapper == nil {
-		fname := uri.Filename()
-		content, err := ioutil.ReadFile(fname)
+		content, err := ioutil.ReadFile(uri.Filename())
 		if err != nil {
 			file.err = fmt.Errorf("getFile: %v: %v", uri, err)
 			return file
 		}
-		f := c.fset.AddFile(fname, -1, len(content))
-		f.SetLinesForContent(content)
-		file.mapper = &protocol.ColumnMapper{
-			URI:     uri,
-			TokFile: f,
-			Content: content,
-		}
+		file.mapper = protocol.NewMapper(uri, content)
 	}
 	return file
 }
@@ -561,19 +629,16 @@ func (c *cmdClient) getFile(ctx context.Context, uri span.URI) *cmdFile {
 func (c *cmdClient) openFile(ctx context.Context, uri span.URI) *cmdFile {
 	c.filesMu.Lock()
 	defer c.filesMu.Unlock()
-
-	file := c.getFile(ctx, uri)
-	if file.err != nil || file.open {
-		return file
-	}
-	file.open = true
-	return file
+	return c.getFile(ctx, uri)
 }
 
-func (c *connection) openFile(ctx context.Context, uri span.URI) *cmdFile {
-	file := c.Client.openFile(ctx, uri)
+// TODO(adonovan): provide convenience helpers to:
+// - map a (URI, protocol.Range) to a MappedRange;
+// - parse a command-line argument to a MappedRange.
+func (c *connection) openFile(ctx context.Context, uri span.URI) (*cmdFile, error) {
+	file := c.client.openFile(ctx, uri)
 	if file.err != nil {
-		return file
+		return nil, file.err
 	}
 
 	p := &protocol.DidOpenTextDocumentParams{
@@ -585,9 +650,11 @@ func (c *connection) openFile(ctx context.Context, uri span.URI) *cmdFile {
 		},
 	}
 	if err := c.Server.DidOpen(ctx, p); err != nil {
+		// TODO(adonovan): is this assignment concurrency safe?
 		file.err = fmt.Errorf("%v: %v", uri, err)
+		return nil, file.err
 	}
-	return file
+	return file, nil
 }
 
 func (c *connection) semanticTokens(ctx context.Context, p *protocol.SemanticTokensRangeParams) (*protocol.SemanticTokens, error) {
@@ -604,22 +671,22 @@ func (c *connection) diagnoseFiles(ctx context.Context, files []span.URI) error
 	for _, file := range files {
 		untypedFiles = append(untypedFiles, string(file))
 	}
-	c.Client.diagnosticsMu.Lock()
-	defer c.Client.diagnosticsMu.Unlock()
+	c.client.diagnosticsMu.Lock()
+	defer c.client.diagnosticsMu.Unlock()
 
-	c.Client.diagnosticsDone = make(chan struct{})
+	c.client.diagnosticsDone = make(chan struct{})
 	_, err := c.Server.NonstandardRequest(ctx, "gopls/diagnoseFiles", map[string]interface{}{"files": untypedFiles})
 	if err != nil {
-		close(c.Client.diagnosticsDone)
+		close(c.client.diagnosticsDone)
 		return err
 	}
 
-	<-c.Client.diagnosticsDone
+	<-c.client.diagnosticsDone
 	return nil
 }
 
 func (c *connection) terminate(ctx context.Context) {
-	if strings.HasPrefix(c.Client.app.Remote, "internal@") {
+	if strings.HasPrefix(c.client.app.Remote, "internal@") {
 		// internal connections need to be left alive for the next test
 		return
 	}
