@@ -15,13 +15,17 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
+	"runtime/pprof"
 	"sort"
 	"strings"
 	"time"
 
 	"golang.org/x/mod/modfile"
 	"golang.org/x/tools/go/ast/astutil"
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/govulncheck"
+	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/debug"
 	"golang.org/x/tools/gopls/internal/lsp/progress"
@@ -31,10 +35,14 @@ import (
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
+	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/xcontext"
 )
 
 func (s *Server) executeCommand(ctx context.Context, params *protocol.ExecuteCommandParams) (interface{}, error) {
+	ctx, done := event.Start(ctx, "lsp.Server.executeCommand")
+	defer done()
+
 	var found bool
 	for _, name := range s.session.Options().SupportedCommands {
 		if name == params.Command {
@@ -63,6 +71,7 @@ type commandConfig struct {
 	async       bool                 // whether to run the command asynchronously. Async commands can only return errors.
 	requireSave bool                 // whether all files must be saved for the command to work
 	progress    string               // title to use for progress reporting. If empty, no progress will be reported.
+	forView     string               // view to resolve to a snapshot; incompatible with forURI
 	forURI      protocol.DocumentURI // URI to resolve to a snapshot. If unset, snapshot will be nil.
 }
 
@@ -70,9 +79,9 @@ type commandConfig struct {
 // be populated, depending on which configuration is set. See comments in-line
 // for details.
 type commandDeps struct {
-	snapshot source.Snapshot            // present if cfg.forURI was set
-	fh       source.VersionedFileHandle // present if cfg.forURI was set
-	work     *progress.WorkDone         // present cfg.progress was set
+	snapshot source.Snapshot    // present if cfg.forURI was set
+	fh       source.FileHandle  // present if cfg.forURI was set
+	work     *progress.WorkDone // present cfg.progress was set
 }
 
 type commandFunc func(context.Context, commandDeps) error
@@ -97,6 +106,9 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 		}
 	}
 	var deps commandDeps
+	if cfg.forURI != "" && cfg.forView != "" {
+		return bug.Errorf("internal error: forURI=%q, forView=%q", cfg.forURI, cfg.forView)
+	}
 	if cfg.forURI != "" {
 		var ok bool
 		var release func()
@@ -108,6 +120,17 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 			}
 			return fmt.Errorf("invalid file URL: %v", cfg.forURI)
 		}
+	} else if cfg.forView != "" {
+		view, err := c.s.session.View(cfg.forView)
+		if err != nil {
+			return err
+		}
+		var release func()
+		deps.snapshot, release, err = view.Snapshot()
+		if err != nil {
+			return err
+		}
+		defer release()
 	}
 	ctx, cancel := context.WithCancel(xcontext.Detach(ctx))
 	if cfg.progress != "" {
@@ -154,8 +177,9 @@ func (c *commandHandler) ApplyFix(ctx context.Context, args command.ApplyFixArgs
 		if err != nil {
 			return err
 		}
-		var changes []protocol.DocumentChanges
+		changes := []protocol.DocumentChanges{} // must be a slice
 		for _, edit := range edits {
+			edit := edit
 			changes = append(changes, protocol.DocumentChanges{
 				TextDocumentEdit: &edit,
 			})
@@ -198,7 +222,7 @@ func (c *commandHandler) CheckUpgrades(ctx context.Context, args command.CheckUp
 		}
 		deps.snapshot.View().RegisterModuleUpgrades(args.URI.SpanURI(), upgrades)
 		// Re-diagnose the snapshot to publish the new module diagnostics.
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false)
+		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
 		return nil
 	})
 }
@@ -223,13 +247,13 @@ func (c *commandHandler) ResetGoModDiagnostics(ctx context.Context, args command
 			c.s.clearDiagnosticSource(modCheckUpgradesSource)
 		}
 
-		if args.DiagnosticSource == "" || args.DiagnosticSource == string(source.Vulncheck) {
+		if args.DiagnosticSource == "" || args.DiagnosticSource == string(source.Govulncheck) {
 			deps.snapshot.View().SetVulnerabilities(args.URI.SpanURI(), nil)
 			c.s.clearDiagnosticSource(modVulncheckSource)
 		}
 
 		// Re-diagnose the snapshot to remove the diagnostics.
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false)
+		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
 		return nil
 	})
 }
@@ -337,10 +361,9 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 		progress: "Removing dependency",
 		forURI:   args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		// If the module is tidied apart from the one unused diagnostic, we can
-		// run `go get module@none`, and then run `go mod tidy`. Otherwise, we
-		// must make textual edits.
-		// TODO(rstambler): In Go 1.17+, we will be able to use the go command
+		// See the documentation for OnlyDiagnostic.
+		//
+		// TODO(rfindley): In Go 1.17+, we will be able to use the go command
 		// without checking if the module is tidy.
 		if args.OnlyDiagnostic {
 			return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI.SpanURI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
@@ -370,7 +393,7 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 									URI: protocol.URIFromSpanURI(deps.fh.URI()),
 								},
 							},
-							Edits: edits,
+							Edits: nonNilSliceTextEdit(edits),
 						},
 					},
 				},
@@ -432,15 +455,11 @@ func (c *commandHandler) RunTests(ctx context.Context, args command.RunTestsArgs
 
 func (c *commandHandler) runTests(ctx context.Context, snapshot source.Snapshot, work *progress.WorkDone, uri protocol.DocumentURI, tests, benchmarks []string) error {
 	// TODO: fix the error reporting when this runs async.
-	metas, err := snapshot.MetadataForFile(ctx, uri.SpanURI())
+	meta, err := source.NarrowestMetadataForFile(ctx, snapshot, uri.SpanURI())
 	if err != nil {
 		return err
 	}
-	metas = source.RemoveIntermediateTestVariants(metas)
-	if len(metas) == 0 {
-		return fmt.Errorf("package could not be found for file: %s", uri.SpanURI().Filename())
-	}
-	pkgPath := string(metas[0].ForTest)
+	pkgPath := string(meta.ForTest)
 
 	// create output
 	buf := &bytes.Buffer{}
@@ -573,47 +592,35 @@ func (s *Server) runGoModUpdateCommands(ctx context.Context, snapshot source.Sna
 	}
 	modURI := snapshot.GoModForFile(uri)
 	sumURI := span.URIFromPath(strings.TrimSuffix(modURI.Filename(), ".mod") + ".sum")
-	modEdits, err := applyFileEdits(ctx, snapshot, modURI, newModBytes)
+	modEdits, err := collectFileEdits(ctx, snapshot, modURI, newModBytes)
 	if err != nil {
 		return err
 	}
-	sumEdits, err := applyFileEdits(ctx, snapshot, sumURI, newSumBytes)
+	sumEdits, err := collectFileEdits(ctx, snapshot, sumURI, newSumBytes)
 	if err != nil {
 		return err
 	}
-	changes := append(sumEdits, modEdits...)
-	if len(changes) == 0 {
-		return nil
-	}
-	var documentChanges []protocol.DocumentChanges
-	for _, change := range changes {
-		documentChanges = append(documentChanges, protocol.DocumentChanges{
-			TextDocumentEdit: &change,
-		})
-	}
-	response, err := s.client.ApplyEdit(ctx, &protocol.ApplyWorkspaceEditParams{
-		Edit: protocol.WorkspaceEdit{
-			DocumentChanges: documentChanges,
-		},
-	})
-	if err != nil {
-		return err
-	}
-	if !response.Applied {
-		return fmt.Errorf("edits not applied because of %s", response.FailureReason)
-	}
-	return nil
+	return applyFileEdits(ctx, s.client, append(sumEdits, modEdits...))
 }
 
-func applyFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.URI, newContent []byte) ([]protocol.TextDocumentEdit, error) {
-	fh, err := snapshot.GetVersionedFile(ctx, uri)
+// collectFileEdits collects any file edits required to transform the snapshot
+// file specified by uri to the provided new content.
+//
+// If the file is not open, collectFileEdits simply writes the new content to
+// disk.
+//
+// TODO(rfindley): fix this API asymmetry. It should be up to the caller to
+// write the file or apply the edits.
+func collectFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.URI, newContent []byte) ([]protocol.TextDocumentEdit, error) {
+	fh, err := snapshot.ReadFile(ctx, uri)
 	if err != nil {
 		return nil, err
 	}
-	oldContent, err := fh.Read()
+	oldContent, err := fh.Content()
 	if err != nil && !os.IsNotExist(err) {
 		return nil, err
 	}
+
 	if bytes.Equal(oldContent, newContent) {
 		return nil, nil
 	}
@@ -626,7 +633,7 @@ func applyFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.URI,
 		return nil, err
 	}
 
-	m := protocol.NewColumnMapper(fh.URI(), oldContent)
+	m := protocol.NewMapper(fh.URI(), oldContent)
 	diff := snapshot.View().Options().ComputeEdits(string(oldContent), string(newContent))
 	edits, err := source.ToProtocolEdits(m, diff)
 	if err != nil {
@@ -643,6 +650,31 @@ func applyFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.URI,
 	}}, nil
 }
 
+func applyFileEdits(ctx context.Context, cli protocol.Client, edits []protocol.TextDocumentEdit) error {
+	if len(edits) == 0 {
+		return nil
+	}
+	documentChanges := []protocol.DocumentChanges{} // must be a slice
+	for _, change := range edits {
+		change := change
+		documentChanges = append(documentChanges, protocol.DocumentChanges{
+			TextDocumentEdit: &change,
+		})
+	}
+	response, err := cli.ApplyEdit(ctx, &protocol.ApplyWorkspaceEditParams{
+		Edit: protocol.WorkspaceEdit{
+			DocumentChanges: documentChanges,
+		},
+	})
+	if err != nil {
+		return err
+	}
+	if !response.Applied {
+		return fmt.Errorf("edits not applied because of %s", response.FailureReason)
+	}
+	return nil
+}
+
 func runGoGetModule(invoke func(...string) (*bytes.Buffer, error), addRequire bool, args []string) error {
 	if addRequire {
 		if err := addModuleRequire(invoke, args); err != nil {
@@ -697,49 +729,19 @@ func (c *commandHandler) ToggleGCDetails(ctx context.Context, args command.URIAr
 		progress:    "Toggling GC Details",
 		forURI:      args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		metas, err := deps.snapshot.MetadataForFile(ctx, deps.fh.URI())
+		meta, err := source.NarrowestMetadataForFile(ctx, deps.snapshot, deps.fh.URI())
 		if err != nil {
 			return err
 		}
-		id := metas[0].ID // 0 => narrowest package
 		c.s.gcOptimizationDetailsMu.Lock()
-		if _, ok := c.s.gcOptimizationDetails[id]; ok {
-			delete(c.s.gcOptimizationDetails, id)
+		if _, ok := c.s.gcOptimizationDetails[meta.ID]; ok {
+			delete(c.s.gcOptimizationDetails, meta.ID)
 			c.s.clearDiagnosticSource(gcDetailsSource)
 		} else {
-			c.s.gcOptimizationDetails[id] = struct{}{}
+			c.s.gcOptimizationDetails[meta.ID] = struct{}{}
 		}
 		c.s.gcOptimizationDetailsMu.Unlock()
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false)
-		return nil
-	})
-}
-
-func (c *commandHandler) GenerateGoplsMod(ctx context.Context, args command.URIArg) error {
-	// TODO: go back to using URI
-	return c.run(ctx, commandConfig{
-		requireSave: true,
-		progress:    "Generating gopls.mod",
-	}, func(ctx context.Context, deps commandDeps) error {
-		views := c.s.session.Views()
-		if len(views) != 1 {
-			return fmt.Errorf("cannot resolve view: have %d views", len(views))
-		}
-		v := views[0]
-		snapshot, release := v.Snapshot(ctx)
-		defer release()
-		modFile, err := snapshot.BuildGoplsMod(ctx)
-		if err != nil {
-			return fmt.Errorf("getting workspace mod file: %w", err)
-		}
-		content, err := modFile.Format()
-		if err != nil {
-			return fmt.Errorf("formatting mod file: %w", err)
-		}
-		filename := filepath.Join(v.Folder().Filename(), "gopls.mod")
-		if err := ioutil.WriteFile(filename, content, 0644); err != nil {
-			return fmt.Errorf("writing mod file: %w", err)
-		}
+		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
 		return nil
 	})
 }
@@ -764,7 +766,7 @@ func (c *commandHandler) ListImports(ctx context.Context, args command.URIArg) (
 	err := c.run(ctx, commandConfig{
 		forURI: args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		fh, err := deps.snapshot.GetFile(ctx, args.URI.SpanURI())
+		fh, err := deps.snapshot.ReadFile(ctx, args.URI.SpanURI())
 		if err != nil {
 			return err
 		}
@@ -772,7 +774,8 @@ func (c *commandHandler) ListImports(ctx context.Context, args command.URIArg) (
 		if err != nil {
 			return err
 		}
-		for _, group := range astutil.Imports(deps.snapshot.FileSet(), pgf.File) {
+		fset := tokeninternal.FileSetFor(pgf.Tok)
+		for _, group := range astutil.Imports(fset, pgf.File) {
 			for _, imp := range group {
 				if imp.Path == nil {
 					continue
@@ -787,14 +790,11 @@ func (c *commandHandler) ListImports(ctx context.Context, args command.URIArg) (
 				})
 			}
 		}
-		metas, err := deps.snapshot.MetadataForFile(ctx, args.URI.SpanURI())
+		meta, err := source.NarrowestMetadataForFile(ctx, deps.snapshot, args.URI.SpanURI())
 		if err != nil {
 			return err // e.g. cancelled
 		}
-		if len(metas) == 0 {
-			return fmt.Errorf("no package containing %v", args.URI.SpanURI())
-		}
-		for pkgPath := range metas[0].DepsByPkgPath { // 0 => narrowest package
+		for pkgPath := range meta.DepsByPkgPath {
 			result.PackageImports = append(result.PackageImports,
 				command.PackageImport{Path: string(pkgPath)})
 		}
@@ -840,6 +840,49 @@ func (c *commandHandler) StartDebugging(ctx context.Context, args command.Debugg
 		return result, fmt.Errorf("starting debug server: %w", err)
 	}
 	result.URLs = []string{"http://" + listenedAddr}
+	openClientBrowser(ctx, c.s.client, result.URLs[0])
+	return result, nil
+}
+
+func (c *commandHandler) StartProfile(ctx context.Context, args command.StartProfileArgs) (result command.StartProfileResult, _ error) {
+	file, err := os.CreateTemp("", "gopls-profile-*")
+	if err != nil {
+		return result, fmt.Errorf("creating temp profile file: %v", err)
+	}
+
+	c.s.ongoingProfileMu.Lock()
+	defer c.s.ongoingProfileMu.Unlock()
+
+	if c.s.ongoingProfile != nil {
+		file.Close() // ignore error
+		return result, fmt.Errorf("profile already started (for %q)", c.s.ongoingProfile.Name())
+	}
+
+	if err := pprof.StartCPUProfile(file); err != nil {
+		file.Close() // ignore error
+		return result, fmt.Errorf("starting profile: %v", err)
+	}
+
+	c.s.ongoingProfile = file
+	return result, nil
+}
+
+func (c *commandHandler) StopProfile(ctx context.Context, args command.StopProfileArgs) (result command.StopProfileResult, _ error) {
+	c.s.ongoingProfileMu.Lock()
+	defer c.s.ongoingProfileMu.Unlock()
+
+	prof := c.s.ongoingProfile
+	c.s.ongoingProfile = nil
+
+	if prof == nil {
+		return result, fmt.Errorf("no ongoing profile")
+	}
+
+	pprof.StopCPUProfile()
+	if err := prof.Close(); err != nil {
+		return result, fmt.Errorf("closing profile file: %v", err)
+	}
+	result.File = prof.Name()
 	return result, nil
 }
 
@@ -942,7 +985,7 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 		result.AsOf = time.Now()
 		deps.snapshot.View().SetVulnerabilities(args.URI.SpanURI(), &result)
 
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false)
+		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
 		vulns := result.Vulns
 		affecting := make([]string, 0, len(vulns))
 		for _, v := range vulns {
@@ -972,3 +1015,220 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 		return command.RunVulncheckResult{Token: token}, nil
 	}
 }
+
+// MemStats implements the MemStats command. It returns an error as a
+// future-proof API, but the resulting error is currently always nil.
+func (c *commandHandler) MemStats(ctx context.Context) (command.MemStatsResult, error) {
+	// GC a few times for stable results.
+	runtime.GC()
+	runtime.GC()
+	runtime.GC()
+	var m runtime.MemStats
+	runtime.ReadMemStats(&m)
+	return command.MemStatsResult{
+		HeapAlloc:  m.HeapAlloc,
+		HeapInUse:  m.HeapInuse,
+		TotalAlloc: m.TotalAlloc,
+	}, nil
+}
+
+// WorkspaceStats implements the WorkspaceStats command, reporting information
+// about the current state of the loaded workspace for the current session.
+func (c *commandHandler) WorkspaceStats(ctx context.Context) (command.WorkspaceStatsResult, error) {
+	var res command.WorkspaceStatsResult
+	res.Files.Total, res.Files.Largest, res.Files.Errs = c.s.session.Cache().FileStats()
+
+	for _, view := range c.s.session.Views() {
+		vs, err := collectViewStats(ctx, view)
+		if err != nil {
+			return res, err
+		}
+		res.Views = append(res.Views, vs)
+	}
+	return res, nil
+}
+
+func collectViewStats(ctx context.Context, view *cache.View) (command.ViewStats, error) {
+	s, release, err := view.Snapshot()
+	if err != nil {
+		return command.ViewStats{}, err
+	}
+	defer release()
+
+	allMD, err := s.AllMetadata(ctx)
+	if err != nil {
+		return command.ViewStats{}, err
+	}
+	allPackages := collectPackageStats(allMD)
+
+	wsMD, err := s.WorkspaceMetadata(ctx)
+	if err != nil {
+		return command.ViewStats{}, err
+	}
+	workspacePackages := collectPackageStats(wsMD)
+
+	var ids []source.PackageID
+	for _, m := range wsMD {
+		ids = append(ids, m.ID)
+	}
+
+	diags, err := s.PackageDiagnostics(ctx, ids...)
+	if err != nil {
+		return command.ViewStats{}, err
+	}
+
+	ndiags := 0
+	for _, d := range diags {
+		ndiags += len(d)
+	}
+
+	return command.ViewStats{
+		GoCommandVersion:  view.GoVersionString(),
+		AllPackages:       allPackages,
+		WorkspacePackages: workspacePackages,
+		Diagnostics:       ndiags,
+	}, nil
+}
+
+func collectPackageStats(md []*source.Metadata) command.PackageStats {
+	var stats command.PackageStats
+	stats.Packages = len(md)
+	modules := make(map[string]bool)
+
+	for _, m := range md {
+		n := len(m.CompiledGoFiles)
+		stats.CompiledGoFiles += n
+		if n > stats.LargestPackage {
+			stats.LargestPackage = n
+		}
+		if m.Module != nil {
+			modules[m.Module.Path] = true
+		}
+	}
+	stats.Modules = len(modules)
+
+	return stats
+}
+
+// RunGoWorkCommand invokes `go work <args>` with the provided arguments.
+//
+// args.InitFirst controls whether to first run `go work init`. This allows a
+// single command to both create and recursively populate a go.work file -- as
+// of writing there is no `go work init -r`.
+//
+// Some thought went into implementing this command. Unlike the go.mod commands
+// above, this command simply invokes the go command and relies on the client
+// to notify gopls of file changes via didChangeWatchedFile notifications.
+// We could instead run these commands with GOWORK set to a temp file, but that
+// poses the following problems:
+//   - directory locations in the resulting temp go.work file will be computed
+//     relative to the directory containing that go.work. If the go.work is in a
+//     tempdir, the directories will need to be translated to/from that dir.
+//   - it would be simpler to use a temp go.work file in the workspace
+//     directory, or whichever directory contains the real go.work file, but
+//     that sets a bad precedent of writing to a user-owned directory. We
+//     shouldn't start doing that.
+//   - Sending workspace edits to create a go.work file would require using
+//     the CreateFile resource operation, which would need to be tested in every
+//     client as we haven't used it before. We don't have time for that right
+//     now.
+//
+// Therefore, we simply require that the current go.work file is saved (if it
+// exists), and delegate to the go command.
+func (c *commandHandler) RunGoWorkCommand(ctx context.Context, args command.RunGoWorkArgs) error {
+	return c.run(ctx, commandConfig{
+		progress: "Running go work command",
+		forView:  args.ViewID,
+	}, func(ctx context.Context, deps commandDeps) (runErr error) {
+		snapshot := deps.snapshot
+		view := snapshot.View().(*cache.View)
+		viewDir := view.Folder().Filename()
+
+		// If the user has explicitly set GOWORK=off, we should warn them
+		// explicitly and avoid potentially misleading errors below.
+		goworkURI, off := view.GOWORK()
+		if off {
+			return fmt.Errorf("cannot modify go.work files when GOWORK=off")
+		}
+		gowork := goworkURI.Filename()
+
+		if goworkURI != "" {
+			fh, err := snapshot.ReadFile(ctx, goworkURI)
+			if err != nil {
+				return fmt.Errorf("reading current go.work file: %v", err)
+			}
+			if !fh.Saved() {
+				return fmt.Errorf("must save workspace file %s before running go work commands", goworkURI)
+			}
+		} else {
+			if !args.InitFirst {
+				// If go.work does not exist, we should have detected that and asked
+				// for InitFirst.
+				return bug.Errorf("internal error: cannot run go work command: required go.work file not found")
+			}
+			gowork = filepath.Join(viewDir, "go.work")
+			if err := c.invokeGoWork(ctx, viewDir, gowork, []string{"init"}); err != nil {
+				return fmt.Errorf("running `go work init`: %v", err)
+			}
+		}
+
+		return c.invokeGoWork(ctx, viewDir, gowork, args.Args)
+	})
+}
+
+func (c *commandHandler) invokeGoWork(ctx context.Context, viewDir, gowork string, args []string) error {
+	inv := gocommand.Invocation{
+		Verb:       "work",
+		Args:       args,
+		WorkingDir: viewDir,
+		Env:        append(os.Environ(), fmt.Sprintf("GOWORK=%s", gowork)),
+	}
+	if _, err := c.s.session.GoCommandRunner().Run(ctx, inv); err != nil {
+		return fmt.Errorf("running go work command: %v", err)
+	}
+	return nil
+}
+
+// openClientBrowser causes the LSP client to open the specified URL
+// in an external browser.
+func openClientBrowser(ctx context.Context, cli protocol.Client, url protocol.URI) {
+	showDocumentImpl(ctx, cli, url, nil)
+}
+
+// openClientEditor causes the LSP client to open the specified document
+// and select the indicated range.
+func openClientEditor(ctx context.Context, cli protocol.Client, loc protocol.Location) {
+	showDocumentImpl(ctx, cli, protocol.URI(loc.URI), &loc.Range)
+}
+
+func showDocumentImpl(ctx context.Context, cli protocol.Client, url protocol.URI, rangeOpt *protocol.Range) {
+	// In principle we shouldn't send a showDocument request to a
+	// client that doesn't support it, as reported by
+	// ShowDocumentClientCapabilities. But even clients that do
+	// support it may defer the real work of opening the document
+	// asynchronously, to avoid deadlocks due to rentrancy.
+	//
+	// For example: client sends request to server; server sends
+	// showDocument to client; client opens editor; editor causes
+	// new RPC to be sent to server, which is still busy with
+	// previous request. (This happens in eglot.)
+	//
+	// So we can't rely on the success/failure information.
+	// That's the reason this function doesn't return an error.
+
+	// "External" means run the system-wide handler (e.g. open(1)
+	// on macOS or xdg-open(1) on Linux) for this URL, ignoring
+	// TakeFocus and Selection. Note that this may still end up
+	// opening the same editor (e.g. VSCode) for a file: URL.
+	res, err := cli.ShowDocument(ctx, &protocol.ShowDocumentParams{
+		URI:       url,
+		External:  rangeOpt == nil,
+		TakeFocus: true,
+		Selection: rangeOpt, // optional
+	})
+	if err != nil {
+		event.Error(ctx, "client.showDocument: %v", err)
+	} else if res != nil && !res.Success {
+		event.Log(ctx, fmt.Sprintf("client declined to open document %v", url))
+	}
+}
