@@ -11,12 +11,12 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"sort"
 	"strings"
 	"sync"
 	"time"
 
-	"golang.org/x/sync/errgroup"
-	"golang.org/x/tools/gopls/internal/lsp/debug/log"
+	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/lsp/mod"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
@@ -25,21 +25,30 @@ import (
 	"golang.org/x/tools/gopls/internal/span"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/tag"
-	"golang.org/x/tools/internal/xcontext"
 )
 
+// TODO(rfindley): simplify this very complicated logic for publishing
+// diagnostics. While doing so, ensure that we can test subtle logic such as
+// for multi-pass diagnostics.
+
 // diagnosticSource differentiates different sources of diagnostics.
+//
+// Diagnostics from the same source overwrite each other, whereas diagnostics
+// from different sources do not. Conceptually, the server state is a mapping
+// from diagnostics source to a set of diagnostics, and each storeDiagnostics
+// operation updates one entry of that mapping.
 type diagnosticSource int
 
 const (
-	modSource diagnosticSource = iota
+	modParseSource diagnosticSource = iota
+	modTidySource
 	gcDetailsSource
 	analysisSource
 	typeCheckSource
 	orphanedSource
 	workSource
 	modCheckUpgradesSource
-	modVulncheckSource
+	modVulncheckSource // source.Govulncheck + source.Vulncheck
 )
 
 // A diagnosticReport holds results for a single diagnostic source.
@@ -79,13 +88,15 @@ type fileReports struct {
 	mustPublish bool
 
 	// The last stored diagnostics for each diagnostic source.
-	reports map[diagnosticSource]diagnosticReport
+	reports map[diagnosticSource]*diagnosticReport
 }
 
 func (d diagnosticSource) String() string {
 	switch d {
-	case modSource:
-		return "FromSource"
+	case modParseSource:
+		return "FromModParse"
+	case modTidySource:
+		return "FromModTidy"
 	case gcDetailsSource:
 		return "FromGCDetails"
 	case analysisSource:
@@ -106,46 +117,69 @@ func (d diagnosticSource) String() string {
 }
 
 // hashDiagnostics computes a hash to identify diags.
+//
+// hashDiagnostics mutates its argument (via sorting).
 func hashDiagnostics(diags ...*source.Diagnostic) string {
+	if len(diags) == 0 {
+		return emptyDiagnosticsHash
+	}
+	return computeDiagnosticHash(diags...)
+}
+
+// opt: pre-computed hash for empty diagnostics
+var emptyDiagnosticsHash = computeDiagnosticHash()
+
+// computeDiagnosticHash should only be called from hashDiagnostics.
+//
+// TODO(rfindley): this should use source.Hash.
+func computeDiagnosticHash(diags ...*source.Diagnostic) string {
 	source.SortDiagnostics(diags)
 	h := sha256.New()
 	for _, d := range diags {
 		for _, t := range d.Tags {
-			fmt.Fprintf(h, "%s", t)
+			fmt.Fprintf(h, "tag: %s\n", t)
 		}
 		for _, r := range d.Related {
-			fmt.Fprintf(h, "%s%s%s", r.URI, r.Message, r.Range)
+			fmt.Fprintf(h, "related: %s %s %s\n", r.Location.URI.SpanURI(), r.Message, r.Location.Range)
+		}
+		fmt.Fprintf(h, "code: %s\n", d.Code)
+		fmt.Fprintf(h, "codeHref: %s\n", d.CodeHref)
+		fmt.Fprintf(h, "message: %s\n", d.Message)
+		fmt.Fprintf(h, "range: %s\n", d.Range)
+		fmt.Fprintf(h, "severity: %s\n", d.Severity)
+		fmt.Fprintf(h, "source: %s\n", d.Source)
+		if d.BundledFixes != nil {
+			fmt.Fprintf(h, "fixes: %s\n", *d.BundledFixes)
 		}
-		fmt.Fprintf(h, "%s%s%s%s", d.Message, d.Range, d.Severity, d.Source)
 	}
 	return fmt.Sprintf("%x", h.Sum(nil))
 }
 
-func (s *Server) diagnoseDetached(snapshot source.Snapshot) {
-	ctx := snapshot.BackgroundContext()
-	ctx = xcontext.Detach(ctx)
-	s.diagnose(ctx, snapshot, false)
-	s.publishDiagnostics(ctx, true, snapshot)
-}
-
 func (s *Server) diagnoseSnapshots(snapshots map[source.Snapshot][]span.URI, onDisk bool) {
 	var diagnosticWG sync.WaitGroup
 	for snapshot, uris := range snapshots {
 		diagnosticWG.Add(1)
 		go func(snapshot source.Snapshot, uris []span.URI) {
 			defer diagnosticWG.Done()
-			s.diagnoseSnapshot(snapshot, uris, onDisk)
+			s.diagnoseSnapshot(snapshot, uris, onDisk, snapshot.View().Options().DiagnosticsDelay)
 		}(snapshot, uris)
 	}
 	diagnosticWG.Wait()
 }
 
-func (s *Server) diagnoseSnapshot(snapshot source.Snapshot, changedURIs []span.URI, onDisk bool) {
+// diagnoseSnapshot computes and publishes diagnostics for the given snapshot.
+//
+// If delay is non-zero, computing diagnostics does not start until after this
+// delay has expired, to allow work to be cancelled by subsequent changes.
+//
+// If changedURIs is non-empty, it is a set of recently changed files that
+// should be diagnosed immediately, and onDisk reports whether these file
+// changes came from a change to on-disk files.
+func (s *Server) diagnoseSnapshot(snapshot source.Snapshot, changedURIs []span.URI, onDisk bool, delay time.Duration) {
 	ctx := snapshot.BackgroundContext()
 	ctx, done := event.Start(ctx, "Server.diagnoseSnapshot", source.SnapshotLabels(snapshot)...)
 	defer done()
 
-	delay := snapshot.View().Options().DiagnosticsDelay
 	if delay > 0 {
 		// 2-phase diagnostics.
 		//
@@ -153,25 +187,18 @@ func (s *Server) diagnoseSnapshot(snapshot source.Snapshot, changedURIs []span.U
 		// does not analyze) packages directly affected by
 		// file modifications.
 		//
-		// The second phase runs analysis on the entire snapshot,
-		// and is debounced by the configured delay.
+		// The second phase runs after the delay, and does everything.
 		s.diagnoseChangedFiles(ctx, snapshot, changedURIs, onDisk)
 		s.publishDiagnostics(ctx, false, snapshot)
 
-		// We debounce diagnostics separately for each view, using the snapshot
-		// local ID as logical ordering.
-		//
-		// TODO(rfindley): it would be cleaner to simply put the diagnostic
-		// debouncer on the view, and remove the "key" argument to debouncing.
-		if ok := <-s.diagDebouncer.debounce(snapshot.View().Name(), snapshot.SequenceID(), time.After(delay)); ok {
-			s.diagnose(ctx, snapshot, false)
-			s.publishDiagnostics(ctx, true, snapshot)
+		select {
+		case <-time.After(delay):
+		case <-ctx.Done():
+			return
 		}
-		return
 	}
 
-	// Ignore possible workspace configuration warnings in the normal flow.
-	s.diagnose(ctx, snapshot, false)
+	s.diagnose(ctx, snapshot, analyzeOpenPackages)
 	s.publishDiagnostics(ctx, true, snapshot)
 }
 
@@ -179,11 +206,7 @@ func (s *Server) diagnoseChangedFiles(ctx context.Context, snapshot source.Snaps
 	ctx, done := event.Start(ctx, "Server.diagnoseChangedFiles", source.SnapshotLabels(snapshot)...)
 	defer done()
 
-	// TODO(adonovan): safety: refactor so that group.Go is called
-	// in a second loop, so that if we should later add an early
-	// return to the first loop, we don't leak goroutines.
-	var group errgroup.Group
-	seen := make(map[*source.Metadata]bool)
+	toDiagnose := make(map[source.PackageID]*source.Metadata)
 	for _, uri := range uris {
 		// If the change is only on-disk and the file is not open, don't
 		// directly request its package. It may not be a workspace package.
@@ -197,45 +220,50 @@ func (s *Server) diagnoseChangedFiles(ctx context.Context, snapshot source.Snaps
 		}
 
 		// Don't request type-checking for builtin.go: it's not a real package.
-		if snapshot.IsBuiltin(ctx, uri) {
+		if snapshot.IsBuiltin(uri) {
+			continue
+		}
+
+		// Don't diagnose files that are ignored by `go list` (e.g. testdata).
+		if snapshot.IgnoredFile(uri) {
 			continue
 		}
 
 		// Find all packages that include this file and diagnose them in parallel.
-		metas, err := snapshot.MetadataForFile(ctx, uri)
+		meta, err := source.NarrowestMetadataForFile(ctx, snapshot, uri)
 		if err != nil {
+			if ctx.Err() != nil {
+				return
+			}
 			// TODO(findleyr): we should probably do something with the error here,
 			// but as of now this can fail repeatedly if load fails, so can be too
 			// noisy to log (and we'll handle things later in the slow pass).
 			continue
 		}
-		for _, m := range metas {
-			if m.IsIntermediateTestVariant() {
-				continue
-			}
-			if !seen[m] {
-				seen[m] = true
-				m := m
-				group.Go(func() error {
-					s.diagnosePkg(ctx, snapshot, m, false)
-					return nil // error result is ignored
-				})
-			}
-		}
+		toDiagnose[meta.ID] = meta
 	}
-	group.Wait() // ignore error
+	s.diagnosePkgs(ctx, snapshot, toDiagnose, nil)
 }
 
+// analysisMode parameterizes analysis behavior of a call to diagnosePkgs.
+type analysisMode int
+
+const (
+	analyzeNothing      analysisMode = iota // don't run any analysis
+	analyzeOpenPackages                     // run analysis on packages with open files
+	analyzeEverything                       // run analysis on all packages
+)
+
 // diagnose is a helper function for running diagnostics with a given context.
 // Do not call it directly. forceAnalysis is only true for testing purposes.
-func (s *Server) diagnose(ctx context.Context, snapshot source.Snapshot, forceAnalysis bool) {
+func (s *Server) diagnose(ctx context.Context, snapshot source.Snapshot, analyze analysisMode) {
 	ctx, done := event.Start(ctx, "Server.diagnose", source.SnapshotLabels(snapshot)...)
 	defer done()
 
 	// Wait for a free diagnostics slot.
 	// TODO(adonovan): opt: shouldn't it be the analysis implementation's
 	// job to de-dup and limit resource consumption? In any case this
-	// this function spends most its time waiting for awaitLoaded, at
+	// function spends most its time waiting for awaitLoaded, at
 	// least initially.
 	select {
 	case <-ctx.Done():
@@ -247,58 +275,55 @@ func (s *Server) diagnose(ctx context.Context, snapshot source.Snapshot, forceAn
 	}()
 
 	// common code for dispatching diagnostics
-	store := func(dsource diagnosticSource, operation string, diagsByFileID map[source.VersionedFileIdentity][]*source.Diagnostic, err error, merge bool) {
+	store := func(dsource diagnosticSource, operation string, diagsByFile map[span.URI][]*source.Diagnostic, err error, merge bool) {
 		if err != nil {
 			event.Error(ctx, "warning: while "+operation, err, source.SnapshotLabels(snapshot)...)
 		}
-		for id, diags := range diagsByFileID {
-			if id.URI == "" {
+		for uri, diags := range diagsByFile {
+			if uri == "" {
 				event.Error(ctx, "missing URI while "+operation, fmt.Errorf("empty URI"), tag.Directory.Of(snapshot.View().Folder().Filename()))
 				continue
 			}
-			s.storeDiagnostics(snapshot, id.URI, dsource, diags, merge)
+			s.storeDiagnostics(snapshot, uri, dsource, diags, merge)
 		}
 	}
 
-	// Diagnose go.mod upgrades.
-	upgradeReports, upgradeErr := mod.UpgradeDiagnostics(ctx, snapshot)
-	if ctx.Err() != nil {
-		log.Trace.Log(ctx, "diagnose cancelled")
-		return
-	}
-	store(modCheckUpgradesSource, "diagnosing go.mod upgrades", upgradeReports, upgradeErr, true)
+	// Diagnostics below are organized by increasing specificity:
+	//  go.work > mod > mod upgrade > mod vuln > package, etc.
 
 	// Diagnose go.work file.
 	workReports, workErr := work.Diagnostics(ctx, snapshot)
 	if ctx.Err() != nil {
-		log.Trace.Log(ctx, "diagnose cancelled")
 		return
 	}
 	store(workSource, "diagnosing go.work file", workReports, workErr, true)
 
 	// Diagnose go.mod file.
-	// (This step demands type checking of all active packages:
-	// the bottleneck in the startup sequence for a big workspace.)
 	modReports, modErr := mod.Diagnostics(ctx, snapshot)
 	if ctx.Err() != nil {
-		log.Trace.Log(ctx, "diagnose cancelled")
 		return
 	}
-	store(modSource, "diagnosing go.mod file", modReports, modErr, true)
+	store(modParseSource, "diagnosing go.mod file", modReports, modErr, true)
+
+	// Diagnose go.mod upgrades.
+	upgradeReports, upgradeErr := mod.UpgradeDiagnostics(ctx, snapshot)
+	if ctx.Err() != nil {
+		return
+	}
+	store(modCheckUpgradesSource, "diagnosing go.mod upgrades", upgradeReports, upgradeErr, true)
 
 	// Diagnose vulnerabilities.
 	vulnReports, vulnErr := mod.VulnerabilityDiagnostics(ctx, snapshot)
 	if ctx.Err() != nil {
-		log.Trace.Log(ctx, "diagnose cancelled")
 		return
 	}
 	store(modVulncheckSource, "diagnosing vulnerabilities", vulnReports, vulnErr, false)
 
-	activeMetas, activeErr := snapshot.ActiveMetadata(ctx)
-	if s.shouldIgnoreError(ctx, snapshot, activeErr) {
+	workspace, err := snapshot.WorkspaceMetadata(ctx)
+	if s.shouldIgnoreError(ctx, snapshot, err) {
 		return
 	}
-	criticalErr := snapshot.GetCriticalError(ctx)
+	criticalErr := snapshot.CriticalError(ctx)
 	if ctx.Err() != nil { // must check ctx after GetCriticalError
 		return
 	}
@@ -317,102 +342,200 @@ func (s *Server) diagnose(ctx context.Context, snapshot source.Snapshot, forceAn
 
 	// If there are no workspace packages, there is nothing to diagnose and
 	// there are no orphaned files.
-	if len(activeMetas) == 0 {
+	if len(workspace) == 0 {
 		return
 	}
 
-	// Run go/analysis diagnosis of packages in parallel.
-	// TODO(adonovan): opt: it may be more efficient to
-	// have diagnosePkg take a set of packages.
+	var wg sync.WaitGroup // for potentially slow operations below
+
+	// Maybe run go mod tidy (if it has been invalidated).
 	//
-	// TODO(adonovan): opt: since the new analysis driver does its
-	// own type checking, we could strength-reduce pkg to
-	// PackageID and get this step started as soon as the set of
-	// active package IDs are known, without waiting for them to load.
+	// Since go mod tidy can be slow, we run it concurrently to diagnostics.
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		modTidyReports, err := mod.TidyDiagnostics(ctx, snapshot)
+		store(modTidySource, "running go mod tidy", modTidyReports, err, true)
+	}()
+
+	// Run type checking and go/analysis diagnosis of packages in parallel.
 	var (
-		wg   sync.WaitGroup
-		seen = map[span.URI]struct{}{}
+		seen       = map[span.URI]struct{}{}
+		toDiagnose = make(map[source.PackageID]*source.Metadata)
+		toAnalyze  = make(map[source.PackageID]unit)
 	)
-	for _, m := range activeMetas {
+	for _, m := range workspace {
+		var hasNonIgnored, hasOpenFile bool
 		for _, uri := range m.CompiledGoFiles {
 			seen[uri] = struct{}{}
+			if !hasNonIgnored && !snapshot.IgnoredFile(uri) {
+				hasNonIgnored = true
+			}
+			if !hasOpenFile && snapshot.IsOpen(uri) {
+				hasOpenFile = true
+			}
+		}
+		if hasNonIgnored {
+			toDiagnose[m.ID] = m
+			if analyze == analyzeEverything || analyze == analyzeOpenPackages && hasOpenFile {
+				toAnalyze[m.ID] = unit{}
+			}
 		}
-
-		wg.Add(1)
-		go func(m *source.Metadata) {
-			defer wg.Done()
-			s.diagnosePkg(ctx, snapshot, m, forceAnalysis)
-		}(m)
 	}
+
+	wg.Add(1)
+	go func() {
+		s.diagnosePkgs(ctx, snapshot, toDiagnose, toAnalyze)
+		wg.Done()
+	}()
+
 	wg.Wait()
 
 	// Orphaned files.
 	// Confirm that every opened file belongs to a package (if any exist in
 	// the workspace). Otherwise, add a diagnostic to the file.
-	for _, o := range s.session.Overlays() {
-		if _, ok := seen[o.URI()]; ok {
-			continue
+	if diags, err := snapshot.OrphanedFileDiagnostics(ctx); err == nil {
+		for uri, diag := range diags {
+			s.storeDiagnostics(snapshot, uri, orphanedSource, []*source.Diagnostic{diag}, true)
 		}
-		diagnostic := s.checkForOrphanedFile(ctx, snapshot, o)
-		if diagnostic == nil {
-			continue
+	} else {
+		if ctx.Err() == nil {
+			event.Error(ctx, "computing orphaned file diagnostics", err, source.SnapshotLabels(snapshot)...)
 		}
-		s.storeDiagnostics(snapshot, o.URI(), orphanedSource, []*source.Diagnostic{diagnostic}, true)
 	}
 }
 
-func (s *Server) diagnosePkg(ctx context.Context, snapshot source.Snapshot, m *source.Metadata, alwaysAnalyze bool) {
-	ctx, done := event.Start(ctx, "Server.diagnosePkg", append(source.SnapshotLabels(snapshot), tag.Package.Of(string(m.ID)))...)
+// diagnosePkgs type checks packages in toDiagnose, and analyzes packages in
+// toAnalyze, merging their diagnostics. Packages in toAnalyze must be a subset
+// of the packages in toDiagnose.
+//
+// It also implements gc_details diagnostics.
+//
+// TODO(rfindley): revisit handling of analysis gc_details. It may be possible
+// to merge this function with Server.diagnose, thereby avoiding the two layers
+// of concurrent dispatch: as of writing we concurrently run TidyDiagnostics
+// and diagnosePkgs, and diagnosePkgs concurrently runs PackageDiagnostics and
+// analysis.
+func (s *Server) diagnosePkgs(ctx context.Context, snapshot source.Snapshot, toDiagnose map[source.PackageID]*source.Metadata, toAnalyze map[source.PackageID]unit) {
+	ctx, done := event.Start(ctx, "Server.diagnosePkgs", source.SnapshotLabels(snapshot)...)
 	defer done()
-	enableDiagnostics := false
-	includeAnalysis := alwaysAnalyze // only run analyses for packages with open files
-	for _, uri := range m.CompiledGoFiles {
-		enableDiagnostics = enableDiagnostics || !snapshot.IgnoredFile(uri)
-		includeAnalysis = includeAnalysis || snapshot.IsOpen(uri)
-	}
-	// Don't show any diagnostics on ignored files.
-	if !enableDiagnostics {
-		return
-	}
 
-	pkgs, err := snapshot.TypeCheck(ctx, source.TypecheckFull, m.ID)
-	if err != nil {
-		event.Error(ctx, "warning: typecheck failed", err, append(source.SnapshotLabels(snapshot), tag.Package.Of(string(m.ID)))...)
-		return
-	}
-	pkg := pkgs[0]
+	// Analyze and type-check concurrently, since they are independent
+	// operations.
+	var (
+		wg            sync.WaitGroup
+		pkgDiags      map[span.URI][]*source.Diagnostic
+		analysisDiags = make(map[span.URI][]*source.Diagnostic)
+	)
+
+	// Collect package diagnostics.
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		var ids []source.PackageID
+		for id := range toDiagnose {
+			ids = append(ids, id)
+		}
+		var err error
+		pkgDiags, err = snapshot.PackageDiagnostics(ctx, ids...)
+		if err != nil {
+			event.Error(ctx, "warning: diagnostics failed", err, source.SnapshotLabels(snapshot)...)
+		}
+	}()
 
 	// Get diagnostics from analysis framework.
 	// This includes type-error analyzers, which suggest fixes to compiler errors.
-	var analysisDiags map[span.URI][]*source.Diagnostic
-	if includeAnalysis {
-		diags, err := source.Analyze(ctx, snapshot, m.ID, false)
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		diags, err := source.Analyze(ctx, snapshot, toAnalyze, s.progress)
 		if err != nil {
-			event.Error(ctx, "warning: analyzing package", err, append(source.SnapshotLabels(snapshot), tag.Package.Of(string(m.ID)))...)
+			var tagStr string // sorted comma-separated list of package IDs
+			{
+				// TODO(adonovan): replace with a generic map[S]any -> string
+				// function in the tag package, and use  maps.Keys + slices.Sort.
+				keys := make([]string, 0, len(toDiagnose))
+				for id := range toDiagnose {
+					keys = append(keys, string(id))
+				}
+				sort.Strings(keys)
+				tagStr = strings.Join(keys, ",")
+			}
+			event.Error(ctx, "warning: analyzing package", err, append(source.SnapshotLabels(snapshot), tag.Package.Of(tagStr))...)
 			return
 		}
-		analysisDiags = diags
-	}
+		for uri, diags := range diags {
+			analysisDiags[uri] = append(analysisDiags[uri], diags...)
+		}
+	}()
 
-	// For each file, update the server's diagnostics state.
-	for _, cgf := range pkg.CompiledGoFiles() {
-		// builtin.go exists only for documentation purposes and
-		// is not valid Go code. Don't report distracting errors.
-		if snapshot.IsBuiltin(ctx, cgf.URI) {
+	wg.Wait()
+
+	// TODO(rfindley): remove the guards against snapshot.IsBuiltin, after the
+	// gopls@v0.12.0 release. Packages should not be producing diagnostics for
+	// the builtin file: I do not know why this logic existed previously.
+
+	// Merge analysis diagnostics with package diagnostics, and store the
+	// resulting analysis diagnostics.
+	for uri, adiags := range analysisDiags {
+		if snapshot.IsBuiltin(uri) {
+			bug.Reportf("go/analysis reported diagnostics for the builtin file: %v", adiags)
 			continue
 		}
-		var tdiags, adiags []*source.Diagnostic
-		source.CombineDiagnostics(pkg, cgf.URI, analysisDiags, &tdiags, &adiags)
-		s.storeDiagnostics(snapshot, cgf.URI, typeCheckSource, tdiags, true)
-		s.storeDiagnostics(snapshot, cgf.URI, analysisSource, adiags, true)
+		tdiags := pkgDiags[uri]
+		var tdiags2, adiags2 []*source.Diagnostic
+		source.CombineDiagnostics(tdiags, adiags, &tdiags2, &adiags2)
+		pkgDiags[uri] = tdiags2
+		s.storeDiagnostics(snapshot, uri, analysisSource, adiags2, true)
 	}
 
-	// If gc optimization details are requested, add them to the
-	// diagnostic reports.
+	// golang/go#59587: guarantee that we store type-checking diagnostics for every compiled
+	// package file.
+	//
+	// Without explicitly storing empty diagnostics, the eager diagnostics
+	// publication for changed files will not publish anything for files with
+	// empty diagnostics.
+	storedPkgDiags := make(map[span.URI]bool)
+	for _, m := range toDiagnose {
+		for _, uri := range m.CompiledGoFiles {
+			s.storeDiagnostics(snapshot, uri, typeCheckSource, pkgDiags[uri], true)
+			storedPkgDiags[uri] = true
+		}
+	}
+	// Store the package diagnostics.
+	for uri, diags := range pkgDiags {
+		if storedPkgDiags[uri] {
+			continue
+		}
+		// builtin.go exists only for documentation purposes, and is not valid Go code.
+		// Don't report distracting errors
+		if snapshot.IsBuiltin(uri) {
+			bug.Reportf("type checking reported diagnostics for the builtin file: %v", diags)
+			continue
+		}
+		s.storeDiagnostics(snapshot, uri, typeCheckSource, diags, true)
+	}
+
+	// Process requested gc_details diagnostics.
+	//
+	// TODO(rfindley): this could be improved:
+	//   1. This should memoize its results if the package has not changed.
+	//   2. This should not even run gc_details if the package contains unsaved
+	//      files.
+	//   3. See note below about using FindFile.
+	var toGCDetail map[source.PackageID]*source.Metadata
 	s.gcOptimizationDetailsMu.Lock()
-	_, enableGCDetails := s.gcOptimizationDetails[m.ID]
+	for id := range s.gcOptimizationDetails {
+		if m, ok := toDiagnose[id]; ok {
+			if toGCDetail == nil {
+				toGCDetail = make(map[source.PackageID]*source.Metadata)
+			}
+			toGCDetail[id] = m
+		}
+	}
 	s.gcOptimizationDetailsMu.Unlock()
-	if enableGCDetails {
+
+	for _, m := range toGCDetail {
 		gcReports, err := source.GCOptimizationDetails(ctx, snapshot, m)
 		if err != nil {
 			event.Error(ctx, "warning: gc details", err, append(source.SnapshotLabels(snapshot), tag.Package.Of(string(m.ID)))...)
@@ -425,14 +548,17 @@ func (s *Server) diagnosePkg(ctx context.Context, snapshot source.Snapshot, m *s
 		// results. This ensures that the toggling of GC details and clearing of
 		// diagnostics does not race with storing the results here.
 		if enableGCDetails {
-			for id, diags := range gcReports {
-				fh := snapshot.FindFile(id.URI)
+			for uri, diags := range gcReports {
+				// TODO(rfindley): remove the use of FindFile here, and use ReadFile
+				// instead. Isn't it enough to know that the package came from the
+				// snapshot? Any reports should apply to the snapshot.
+				fh := snapshot.FindFile(uri)
 				// Don't publish gc details for unsaved buffers, since the underlying
 				// logic operates on the file on disk.
 				if fh == nil || !fh.Saved() {
 					continue
 				}
-				s.storeDiagnostics(snapshot, id.URI, gcDetailsSource, diags, true)
+				s.storeDiagnostics(snapshot, uri, gcDetailsSource, diags, true)
 			}
 		}
 		s.gcOptimizationDetailsMu.Unlock()
@@ -451,7 +577,7 @@ func (s *Server) mustPublishDiagnostics(uri span.URI) {
 	if s.diagnostics[uri] == nil {
 		s.diagnostics[uri] = &fileReports{
 			publishedHash: hashDiagnostics(), // Hash for 0 diagnostics.
-			reports:       map[diagnosticSource]diagnosticReport{},
+			reports:       map[diagnosticSource]*diagnosticReport{},
 		}
 	}
 	s.diagnostics[uri].mustPublish = true
@@ -460,6 +586,8 @@ func (s *Server) mustPublishDiagnostics(uri span.URI) {
 // storeDiagnostics stores results from a single diagnostic source. If merge is
 // true, it merges results into any existing results for this snapshot.
 //
+// Mutates (sorts) diags.
+//
 // TODO(hyangah): investigate whether we can unconditionally overwrite previous report.diags
 // with the new diags and eliminate the need for the `merge` flag.
 func (s *Server) storeDiagnostics(snapshot source.Snapshot, uri span.URI, dsource diagnosticSource, diags []*source.Diagnostic, merge bool) {
@@ -475,10 +603,14 @@ func (s *Server) storeDiagnostics(snapshot source.Snapshot, uri span.URI, dsourc
 	if s.diagnostics[uri] == nil {
 		s.diagnostics[uri] = &fileReports{
 			publishedHash: hashDiagnostics(), // Hash for 0 diagnostics.
-			reports:       map[diagnosticSource]diagnosticReport{},
+			reports:       map[diagnosticSource]*diagnosticReport{},
 		}
 	}
 	report := s.diagnostics[uri].reports[dsource]
+	if report == nil {
+		report = new(diagnosticReport)
+		s.diagnostics[uri].reports[dsource] = report
+	}
 	// Don't set obsolete diagnostics.
 	if report.snapshotID > snapshot.GlobalID() {
 		return
@@ -490,7 +622,6 @@ func (s *Server) storeDiagnostics(snapshot source.Snapshot, uri span.URI, dsourc
 	for _, d := range diags {
 		report.diags[hashDiagnostics(d)] = d
 	}
-	s.diagnostics[uri].reports[dsource] = report
 }
 
 // clearDiagnosticSource clears all diagnostics for a given source type. It is
@@ -518,7 +649,7 @@ func (s *Server) showCriticalErrorStatus(ctx context.Context, snapshot source.Sn
 	if err != nil {
 		event.Error(ctx, "errors loading workspace", err.MainError, source.SnapshotLabels(snapshot)...)
 		for _, d := range err.Diagnostics {
-			s.storeDiagnostics(snapshot, d.URI, modSource, []*source.Diagnostic{d}, true)
+			s.storeDiagnostics(snapshot, d.URI, modParseSource, []*source.Diagnostic{d}, true)
 		}
 		errMsg = strings.ReplaceAll(err.MainError.Error(), "\n", " ")
 	}
@@ -540,66 +671,6 @@ func (s *Server) showCriticalErrorStatus(ctx context.Context, snapshot source.Sn
 	}
 }
 
-// checkForOrphanedFile checks that the given URIs can be mapped to packages.
-// If they cannot and the workspace is not otherwise unloaded, it also surfaces
-// a warning, suggesting that the user check the file for build tags.
-func (s *Server) checkForOrphanedFile(ctx context.Context, snapshot source.Snapshot, fh source.VersionedFileHandle) *source.Diagnostic {
-	// TODO(rfindley): this function may fail to produce a diagnostic for a
-	// variety of reasons, some of which should probably not be ignored. For
-	// example, should this function be tolerant of the case where fh does not
-	// exist, or does not have a package name?
-	//
-	// It would be better to panic or report a bug in several of the cases below,
-	// so that we can move toward guaranteeing we show the user a meaningful
-	// error whenever it makes sense.
-	if snapshot.View().FileKind(fh) != source.Go {
-		return nil
-	}
-	// builtin files won't have a package, but they are never orphaned.
-	if snapshot.IsBuiltin(ctx, fh.URI()) {
-		return nil
-	}
-
-	// This call has the effect of inserting fh into snapshot.files,
-	// where for better or worse (actually: just worse) it influences
-	// the sets of open, known, and orphaned files.
-	snapshot.GetFile(ctx, fh.URI())
-
-	metas, _ := snapshot.MetadataForFile(ctx, fh.URI())
-	if len(metas) > 0 || ctx.Err() != nil {
-		return nil // no package, or cancelled
-	}
-	// Inv: file does not belong to a package we know about.
-	pgf, err := snapshot.ParseGo(ctx, fh, source.ParseHeader)
-	if err != nil {
-		return nil
-	}
-	if !pgf.File.Name.Pos().IsValid() {
-		return nil
-	}
-	rng, err := pgf.Mapper.PosRange(pgf.File.Name.Pos(), pgf.File.Name.End())
-	if err != nil {
-		return nil
-	}
-	// If the file no longer has a name ending in .go, this diagnostic is wrong
-	if filepath.Ext(fh.URI().Filename()) != ".go" {
-		return nil
-	}
-	// TODO(rstambler): We should be able to parse the build tags in the
-	// file and show a more specific error message. For now, put the diagnostic
-	// on the package declaration.
-	return &source.Diagnostic{
-		URI:      fh.URI(),
-		Range:    rng,
-		Severity: protocol.SeverityWarning,
-		Source:   source.ListError,
-		Message: fmt.Sprintf(`No packages found for open file %s: %v.
-If this file contains build tags, try adding "-tags=<build tag>" to your gopls "buildFlags" configuration (see (https://github.com/golang/tools/blob/master/gopls/doc/settings.md#buildflags-string).
-Otherwise, see the troubleshooting guidelines for help investigating (https://github.com/golang/tools/blob/master/gopls/doc/troubleshooting.md).
-`, fh.URI().Filename(), err),
-	}
-}
-
 // publishDiagnostics collects and publishes any unpublished diagnostic reports.
 func (s *Server) publishDiagnostics(ctx context.Context, final bool, snapshot source.Snapshot) {
 	ctx, done := event.Start(ctx, "Server.publishDiagnostics", source.SnapshotLabels(snapshot)...)
@@ -637,6 +708,7 @@ func (s *Server) publishDiagnostics(ctx context.Context, final bool, snapshot so
 				diags = append(diags, d)
 				reportDiags = append(reportDiags, d)
 			}
+
 			hash := hashDiagnostics(reportDiags...)
 			if hash != report.publishedHash {
 				anyReportsChanged = true
@@ -650,7 +722,6 @@ func (s *Server) publishDiagnostics(ctx context.Context, final bool, snapshot so
 			continue
 		}
 
-		source.SortDiagnostics(diags)
 		hash := hashDiagnostics(diags...)
 		if hash == r.publishedHash && !r.mustPublish {
 			// Update snapshotID to be the latest snapshot for which this diagnostic
@@ -670,15 +741,22 @@ func (s *Server) publishDiagnostics(ctx context.Context, final bool, snapshot so
 			r.publishedHash = hash
 			r.mustPublish = false // diagnostics have been successfully published
 			r.publishedSnapshotID = snapshot.GlobalID()
-			for dsource, hash := range reportHashes {
-				report := r.reports[dsource]
-				report.publishedHash = hash
-				r.reports[dsource] = report
+			// When we publish diagnostics for a file, we must update the
+			// publishedHash for every report, not just the reports that were
+			// published. Eliding a report is equivalent to publishing empty
+			// diagnostics.
+			for dsource, report := range r.reports {
+				if hash, ok := reportHashes[dsource]; ok {
+					report.publishedHash = hash
+				} else {
+					// The report was not (yet) stored for this snapshot. Record that we
+					// published no diagnostics from this source.
+					report.publishedHash = hashDiagnostics()
+				}
 			}
 		} else {
 			if ctx.Err() != nil {
 				// Publish may have failed due to a cancelled context.
-				log.Trace.Log(ctx, "publish cancelled")
 				return
 			}
 			event.Error(ctx, "publishReports: failed to deliver diagnostic", err, tag.URI.Of(uri))
@@ -689,24 +767,15 @@ func (s *Server) publishDiagnostics(ctx context.Context, final bool, snapshot so
 func toProtocolDiagnostics(diagnostics []*source.Diagnostic) []protocol.Diagnostic {
 	reports := []protocol.Diagnostic{}
 	for _, diag := range diagnostics {
-		related := make([]protocol.DiagnosticRelatedInformation, 0, len(diag.Related))
-		for _, rel := range diag.Related {
-			related = append(related, protocol.DiagnosticRelatedInformation{
-				Location: protocol.Location{
-					URI:   protocol.URIFromSpanURI(rel.URI),
-					Range: rel.Range,
-				},
-				Message: rel.Message,
-			})
-		}
 		pdiag := protocol.Diagnostic{
 			// diag.Message might start with \n or \t
 			Message:            strings.TrimSpace(diag.Message),
 			Range:              diag.Range,
 			Severity:           diag.Severity,
 			Source:             string(diag.Source),
-			Tags:               diag.Tags,
-			RelatedInformation: related,
+			Tags:               emptySliceDiagnosticTag(diag.Tags),
+			RelatedInformation: diag.Related,
+			Data:               diag.BundledFixes,
 		}
 		if diag.Code != "" {
 			pdiag.Code = diag.Code
@@ -727,6 +796,8 @@ func (s *Server) shouldIgnoreError(ctx context.Context, snapshot source.Snapshot
 		return true
 	}
 	// If the folder has no Go code in it, we shouldn't spam the user with a warning.
+	// TODO(rfindley): surely it is not correct to walk the folder here just to
+	// suppress diagnostics, every time we compute diagnostics.
 	var hasGo bool
 	_ = filepath.Walk(snapshot.View().Folder().Filename(), func(path string, info os.FileInfo, err error) error {
 		if err != nil {
@@ -762,12 +833,12 @@ func (s *Server) Diagnostics() map[string][]string {
 	return ans
 }
 
-func auxStr(v *source.Diagnostic, d diagnosticReport, typ diagnosticSource) string {
+func auxStr(v *source.Diagnostic, d *diagnosticReport, typ diagnosticSource) string {
 	// Tags? RelatedInformation?
 	msg := fmt.Sprintf("(%s)%q(source:%q,code:%q,severity:%s,snapshot:%d,type:%s)",
 		v.Range, v.Message, v.Source, v.Code, v.Severity, d.snapshotID, typ)
 	for _, r := range v.Related {
-		msg += fmt.Sprintf(" [%s:%s,%q]", r.URI.Filename(), r.Range, r.Message)
+		msg += fmt.Sprintf(" [%s:%s,%q]", r.Location.URI.SpanURI().Filename(), r.Location.Range, r.Message)
 	}
 	return msg
 }
