@@ -10,9 +10,14 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/template"
+	"golang.org/x/tools/internal/event"
+	"golang.org/x/tools/internal/event/tag"
 )
 
 func (s *Server) references(ctx context.Context, params *protocol.ReferenceParams) ([]protocol.Location, error) {
+	ctx, done := event.Start(ctx, "lsp.Server.references", tag.URI.Of(params.TextDocument.URI))
+	defer done()
+
 	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, params.TextDocument.URI, source.UnknownKind)
 	defer release()
 	if !ok {
@@ -21,20 +26,5 @@ func (s *Server) references(ctx context.Context, params *protocol.ReferenceParam
 	if snapshot.View().FileKind(fh) == source.Tmpl {
 		return template.References(ctx, snapshot, fh, params)
 	}
-	references, err := source.References(ctx, snapshot, fh, params.Position, params.Context.IncludeDeclaration)
-	if err != nil {
-		return nil, err
-	}
-	var locations []protocol.Location
-	for _, ref := range references {
-		refRange, err := ref.Range()
-		if err != nil {
-			return nil, err
-		}
-		locations = append(locations, protocol.Location{
-			URI:   protocol.URIFromSpanURI(ref.URI()),
-			Range: refRange,
-		})
-	}
-	return locations, nil
+	return source.References(ctx, snapshot, fh, params.Position, params.Context.IncludeDeclaration)
 }
