@@ -26,6 +26,8 @@ import (
 	"golang.org/x/tools/go/analysis/passes/composite"
 	"golang.org/x/tools/go/analysis/passes/copylock"
 	"golang.org/x/tools/go/analysis/passes/deepequalerrors"
+	"golang.org/x/tools/go/analysis/passes/defers"
+	"golang.org/x/tools/go/analysis/passes/directive"
 	"golang.org/x/tools/go/analysis/passes/errorsas"
 	"golang.org/x/tools/go/analysis/passes/fieldalignment"
 	"golang.org/x/tools/go/analysis/passes/httpresponse"
@@ -49,6 +51,7 @@ import (
 	"golang.org/x/tools/go/analysis/passes/unsafeptr"
 	"golang.org/x/tools/go/analysis/passes/unusedresult"
 	"golang.org/x/tools/go/analysis/passes/unusedwrite"
+	"golang.org/x/tools/gopls/internal/lsp/analysis/deprecated"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/embeddirective"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/fillreturns"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/fillstruct"
@@ -115,23 +118,23 @@ func DefaultOptions() *Options {
 			},
 			UserOptions: UserOptions{
 				BuildOptions: BuildOptions{
-					ExpandWorkspaceToModule:     true,
-					ExperimentalPackageCacheKey: true,
-					MemoryMode:                  ModeNormal,
-					DirectoryFilters:            []string{"-**/node_modules"},
-					TemplateExtensions:          []string{},
-					StandaloneTags:              []string{"ignore"},
+					ExpandWorkspaceToModule: true,
+					MemoryMode:              ModeNormal,
+					DirectoryFilters:        []string{"-**/node_modules"},
+					TemplateExtensions:      []string{},
+					StandaloneTags:          []string{"ignore"},
 				},
 				UIOptions: UIOptions{
 					DiagnosticOptions: DiagnosticOptions{
-						DiagnosticsDelay: 250 * time.Millisecond,
 						Annotations: map[Annotation]bool{
 							Bounds: true,
 							Escape: true,
 							Inline: true,
 							Nil:    true,
 						},
-						Vulncheck: ModeVulncheckOff,
+						Vulncheck:                 ModeVulncheckOff,
+						DiagnosticsDelay:          1 * time.Second,
+						AnalysisProgressReporting: true,
 					},
 					InlayHintOptions: InlayHintOptions{},
 					DocumentationOptions: DocumentationOptions{
@@ -140,9 +143,10 @@ func DefaultOptions() *Options {
 						LinksInHover: true,
 					},
 					NavigationOptions: NavigationOptions{
-						ImportShortcut: Both,
+						ImportShortcut: BothShortcuts,
 						SymbolMatcher:  SymbolFastFuzzy,
 						SymbolStyle:    DynamicSymbols,
+						SymbolScope:    AllSymbolScope,
 					},
 					CompletionOptions: CompletionOptions{
 						Matcher:                        Fuzzy,
@@ -161,13 +165,15 @@ func DefaultOptions() *Options {
 				},
 			},
 			InternalOptions: InternalOptions{
-				LiteralCompletions:      true,
-				TempModfile:             true,
-				CompleteUnimported:      true,
-				CompletionDocumentation: true,
-				DeepCompletion:          true,
-				ChattyDiagnostics:       true,
-				NewDiff:                 "both",
+				LiteralCompletions:          true,
+				TempModfile:                 true,
+				CompleteUnimported:          true,
+				CompletionDocumentation:     true,
+				DeepCompletion:              true,
+				ChattyDiagnostics:           true,
+				NewDiff:                     "both",
+				SubdirWatchPatterns:         SubdirWatchPatternsAuto,
+				ReportAnalysisProgressAfter: 5 * time.Second,
 			},
 			Hooks: Hooks{
 				// TODO(adonovan): switch to new diff.Strings implementation.
@@ -194,9 +200,26 @@ type Options struct {
 	Hooks
 }
 
+// IsAnalyzerEnabled reports whether an analyzer with the given name is
+// enabled.
+//
+// TODO(rfindley): refactor to simplify this function. We no longer need the
+// different categories of analyzer.
+func (opts *Options) IsAnalyzerEnabled(name string) bool {
+	for _, amap := range []map[string]*Analyzer{opts.DefaultAnalyzers, opts.TypeErrorAnalyzers, opts.ConvenienceAnalyzers, opts.StaticcheckAnalyzers} {
+		for _, analyzer := range amap {
+			if analyzer.Analyzer.Name == name && analyzer.IsEnabled(opts) {
+				return true
+			}
+		}
+	}
+	return false
+}
+
 // ClientOptions holds LSP-specific configuration that is provided by the
 // client.
 type ClientOptions struct {
+	ClientInfo                                 *protocol.Msg_XInitializeParams_clientInfo
 	InsertTextFormat                           protocol.InsertTextFormat
 	ConfigurationSupported                     bool
 	DynamicConfigurationSupported              bool
@@ -268,22 +291,6 @@ type BuildOptions struct {
 	// a go.mod file, narrowing the scope to that directory if it exists.
 	ExpandWorkspaceToModule bool `status:"experimental"`
 
-	// ExperimentalWorkspaceModule opts a user into the experimental support
-	// for multi-module workspaces.
-	//
-	// Deprecated: this feature is deprecated and will be removed in a future
-	// version of gopls (https://go.dev/issue/55331).
-	ExperimentalWorkspaceModule bool `status:"experimental"`
-
-	// ExperimentalPackageCacheKey controls whether to use a coarser cache key
-	// for package type information to increase cache hits. This setting removes
-	// the user's environment, build flags, and working directory from the cache
-	// key, which should be a safe change as all relevant inputs into the type
-	// checking pass are already hashed into the key. This is temporarily guarded
-	// by an experiment because caching behavior is subtle and difficult to
-	// comprehensively test.
-	ExperimentalPackageCacheKey bool `status:"experimental"`
-
 	// AllowModfileModifications disables -mod=readonly, allowing imports from
 	// out-of-scope modules. This option will eventually be removed.
 	AllowModfileModifications bool `status:"experimental"`
@@ -442,17 +449,16 @@ type DiagnosticOptions struct {
 	// This option must be set to a valid duration string, for example `"250ms"`.
 	DiagnosticsDelay time.Duration `status:"advanced"`
 
-	// ExperimentalWatchedFileDelay controls the amount of time that gopls waits
-	// for additional workspace/didChangeWatchedFiles notifications to arrive,
-	// before processing all such notifications in a single batch. This is
-	// intended for use by LSP clients that don't support their own batching of
-	// file system notifications.
+	// AnalysisProgressReporting controls whether gopls sends progress
+	// notifications when construction of its index of analysis facts is taking a
+	// long time. Cancelling these notifications will cancel the indexing task,
+	// though it will restart after the next change in the workspace.
 	//
-	// This option must be set to a valid duration string, for example `"100ms"`.
-	//
-	// Deprecated: this setting is deprecated and will be removed in a future
-	// version of gopls (https://go.dev/issue/55332)
-	ExperimentalWatchedFileDelay time.Duration `status:"experimental"`
+	// When a package is opened for the first time and heavyweight analyses such as
+	// staticcheck are enabled, it can take a while to construct the index of
+	// analysis facts for all its dependencies. The index is cached in the
+	// filesystem, so subsequent analysis should be faster.
+	AnalysisProgressReporting bool
 }
 
 type InlayHintOptions struct {
@@ -482,6 +488,13 @@ type NavigationOptions struct {
 	// }
 	// ```
 	SymbolStyle SymbolStyle `status:"advanced"`
+
+	// SymbolScope controls which packages are searched for workspace/symbol
+	// requests. The default value, "workspace", searches only workspace
+	// packages. The legacy behavior, "all", causes all loaded packages to be
+	// searched, including dependencies; this is more expensive and may return
+	// unwanted results.
+	SymbolScope SymbolScope
 }
 
 // UserOptions holds custom Gopls configuration (not part of the LSP) that is
@@ -556,6 +569,9 @@ type Hooks struct {
 // average user. These may be settings used by tests or outdated settings that
 // will soon be deprecated. Some of these settings may not even be configurable
 // by the user.
+//
+// TODO(rfindley): even though these settings are not intended for
+// modification, some of them should be surfaced in our documentation.
 type InternalOptions struct {
 	// LiteralCompletions controls whether literal candidates such as
 	// "&someStruct{}" are offered. Tests disable this flag to simplify
@@ -619,22 +635,62 @@ type InternalOptions struct {
 	// file change. If unset, gopls only reports diagnostics when they change, or
 	// when a file is opened or closed.
 	ChattyDiagnostics bool
+
+	// SubdirWatchPatterns configures the file watching glob patterns registered
+	// by gopls.
+	//
+	// Some clients (namely VS Code) do not send workspace/didChangeWatchedFile
+	// notifications for files contained in a directory when that directory is
+	// deleted:
+	// https://github.com/microsoft/vscode/issues/109754
+	//
+	// In this case, gopls would miss important notifications about deleted
+	// packages. To work around this, gopls registers a watch pattern for each
+	// directory containing Go files.
+	//
+	// Unfortunately, other clients experience performance problems with this
+	// many watch patterns, so there is no single behavior that works well for
+	// all clients.
+	//
+	// The "subdirWatchPatterns" setting allows configuring this behavior. Its
+	// default value of "auto" attempts to guess the correct behavior based on
+	// the client name. We'd love to avoid this specialization, but as described
+	// above there is no single value that works for all clients.
+	//
+	// If any LSP client does not behave well with the default value (for
+	// example, if like VS Code it drops file notifications), please file an
+	// issue.
+	SubdirWatchPatterns SubdirWatchPatterns
+
+	// ReportAnalysisProgressAfter sets the duration for gopls to wait before starting
+	// progress reporting for ongoing go/analysis passes.
+	//
+	// It is intended to be used for testing only.
+	ReportAnalysisProgressAfter time.Duration
 }
 
+type SubdirWatchPatterns string
+
+const (
+	SubdirWatchPatternsOn   SubdirWatchPatterns = "on"
+	SubdirWatchPatternsOff  SubdirWatchPatterns = "off"
+	SubdirWatchPatternsAuto SubdirWatchPatterns = "auto"
+)
+
 type ImportShortcut string
 
 const (
-	Both       ImportShortcut = "Both"
-	Link       ImportShortcut = "Link"
-	Definition ImportShortcut = "Definition"
+	BothShortcuts      ImportShortcut = "Both"
+	LinkShortcut       ImportShortcut = "Link"
+	DefinitionShortcut ImportShortcut = "Definition"
 )
 
 func (s ImportShortcut) ShowLinks() bool {
-	return s == Both || s == Link
+	return s == BothShortcuts || s == LinkShortcut
 }
 
 func (s ImportShortcut) ShowDefinition() bool {
-	return s == Both || s == Definition
+	return s == BothShortcuts || s == DefinitionShortcut
 }
 
 type Matcher string
@@ -645,6 +701,8 @@ const (
 	CaseSensitive   Matcher = "CaseSensitive"
 )
 
+// A SymbolMatcher controls the matching of symbols for workspace/symbol
+// requests.
 type SymbolMatcher string
 
 const (
@@ -654,6 +712,7 @@ const (
 	SymbolCaseSensitive   SymbolMatcher = "CaseSensitive"
 )
 
+// A SymbolStyle controls the formatting of symbols in workspace/symbol results.
 type SymbolStyle string
 
 const (
@@ -670,6 +729,17 @@ const (
 	DynamicSymbols SymbolStyle = "Dynamic"
 )
 
+// A SymbolScope controls the search scope for workspace/symbol requests.
+type SymbolScope string
+
+const (
+	// WorkspaceSymbolScope matches symbols in workspace packages only.
+	WorkspaceSymbolScope SymbolScope = "workspace"
+	// AllSymbolScope matches symbols in any loaded package, including
+	// dependencies.
+	AllSymbolScope SymbolScope = "all"
+)
+
 type HoverKind string
 
 const (
@@ -748,7 +818,8 @@ func SetOptions(options *Options, opts interface{}) OptionResults {
 	return results
 }
 
-func (o *Options) ForClientCapabilities(caps protocol.ClientCapabilities) {
+func (o *Options) ForClientCapabilities(clientName *protocol.Msg_XInitializeParams_clientInfo, caps protocol.ClientCapabilities) {
+	o.ClientInfo = clientName
 	// Check if the client supports snippets in completion items.
 	if caps.Workspace.WorkspaceEdit != nil {
 		o.SupportedResourceOperations = caps.Workspace.WorkspaceEdit.ResourceOperations
@@ -763,15 +834,18 @@ func (o *Options) ForClientCapabilities(caps protocol.ClientCapabilities) {
 	o.DynamicWatchedFilesSupported = caps.Workspace.DidChangeWatchedFiles.DynamicRegistration
 
 	// Check which types of content format are supported by this client.
-	if hover := caps.TextDocument.Hover; len(hover.ContentFormat) > 0 {
+	if hover := caps.TextDocument.Hover; hover != nil && len(hover.ContentFormat) > 0 {
 		o.PreferredContentFormat = hover.ContentFormat[0]
 	}
 	// Check if the client supports only line folding.
-	fr := caps.TextDocument.FoldingRange
-	o.LineFoldingOnly = fr.LineFoldingOnly
+
+	if fr := caps.TextDocument.FoldingRange; fr != nil {
+		o.LineFoldingOnly = fr.LineFoldingOnly
+	}
 	// Check if the client supports hierarchical document symbols.
 	o.HierarchicalDocumentSymbolSupport = caps.TextDocument.DocumentSymbol.HierarchicalDocumentSymbolSupport
-	// Check if the client supports semantic tokens
+
+	// Client's semantic tokens
 	o.SemanticTypes = caps.TextDocument.SemanticTokens.TokenTypes
 	o.SemanticMods = caps.TextDocument.SemanticTokens.TokenModifiers
 	// we don't need Requests, as we support full functionality
@@ -994,6 +1068,14 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 			o.SymbolStyle = SymbolStyle(s)
 		}
 
+	case "symbolScope":
+		if s, ok := result.asOneOf(
+			string(WorkspaceSymbolScope),
+			string(AllSymbolScope),
+		); ok {
+			o.SymbolScope = SymbolScope(s)
+		}
+
 	case "hoverKind":
 		if s, ok := result.asOneOf(
 			string(NoDocumentation),
@@ -1012,7 +1094,7 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 		result.setBool(&o.LinksInHover)
 
 	case "importShortcut":
-		if s, ok := result.asOneOf(string(Both), string(Link), string(Definition)); ok {
+		if s, ok := result.asOneOf(string(BothShortcuts), string(LinkShortcut), string(DefinitionShortcut)); ok {
 			o.ImportShortcut = ImportShortcut(s)
 		}
 
@@ -1100,12 +1182,7 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 		result.setBool(&o.ExperimentalPostfixCompletions)
 
 	case "experimentalWorkspaceModule":
-		const msg = "experimentalWorkspaceModule has been replaced by go workspaces, " +
-			"and will be removed in a future version of gopls (https://go.dev/issue/55331) -- " +
-			"see https://github.com/golang/tools/blob/master/gopls/doc/workspace.md " +
-			"for information on setting up multi-module workspaces using go.work files"
-		result.softErrorf(msg)
-		result.setBool(&o.ExperimentalWorkspaceModule)
+		result.deprecated("")
 
 	case "experimentalTemplateSupport": // TODO(pjw): remove after June 2022
 		result.deprecated("")
@@ -1131,14 +1208,14 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 	case "diagnosticsDelay":
 		result.setDuration(&o.DiagnosticsDelay)
 
+	case "analysisProgressReporting":
+		result.setBool(&o.AnalysisProgressReporting)
+
 	case "experimentalWatchedFileDelay":
-		const msg = "experimentalWatchedFileDelay is deprecated, and will " +
-			"be removed in a future version of gopls (https://go.dev/issue/55332)"
-		result.softErrorf(msg)
-		result.setDuration(&o.ExperimentalWatchedFileDelay)
+		result.deprecated("")
 
 	case "experimentalPackageCacheKey":
-		result.setBool(&o.ExperimentalPackageCacheKey)
+		result.deprecated("")
 
 	case "allowModfileModifications":
 		result.setBool(&o.AllowModfileModifications)
@@ -1162,6 +1239,18 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 	case "chattyDiagnostics":
 		result.setBool(&o.ChattyDiagnostics)
 
+	case "subdirWatchPatterns":
+		if s, ok := result.asOneOf(
+			string(SubdirWatchPatternsOn),
+			string(SubdirWatchPatternsOff),
+			string(SubdirWatchPatternsAuto),
+		); ok {
+			o.SubdirWatchPatterns = SubdirWatchPatterns(s)
+		}
+
+	case "reportAnalysisProgressAfter":
+		result.setDuration(&o.ReportAnalysisProgressAfter)
+
 	// Replaced settings.
 	case "experimentalDisabledAnalyses":
 		result.deprecated("analyses")
@@ -1397,7 +1486,8 @@ func (r *OptionResult) setStringSlice(s *[]string) {
 func typeErrorAnalyzers() map[string]*Analyzer {
 	return map[string]*Analyzer{
 		fillreturns.Analyzer.Name: {
-			Analyzer:   fillreturns.Analyzer,
+			Analyzer: fillreturns.Analyzer,
+			// TODO(rfindley): is SourceFixAll even necessary here? Is that not implied?
 			ActionKind: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix},
 			Enabled:    true,
 		},
@@ -1421,6 +1511,8 @@ func typeErrorAnalyzers() map[string]*Analyzer {
 	}
 }
 
+// TODO(golang/go#61559): remove convenience analyzers now that they are not
+// used from the analysis framework.
 func convenienceAnalyzers() map[string]*Analyzer {
 	return map[string]*Analyzer{
 		fillstruct.Analyzer.Name: {
@@ -1430,10 +1522,14 @@ func convenienceAnalyzers() map[string]*Analyzer {
 			ActionKind: []protocol.CodeActionKind{protocol.RefactorRewrite},
 		},
 		stubmethods.Analyzer.Name: {
-			Analyzer:   stubmethods.Analyzer,
-			ActionKind: []protocol.CodeActionKind{protocol.RefactorRewrite},
-			Fix:        StubMethods,
+			Analyzer: stubmethods.Analyzer,
+			Fix:      StubMethods,
+			Enabled:  true,
+		},
+		infertypeargs.Analyzer.Name: {
+			Analyzer:   infertypeargs.Analyzer,
 			Enabled:    true,
+			ActionKind: []protocol.CodeActionKind{protocol.RefactorRewrite},
 		},
 	}
 }
@@ -1449,6 +1545,9 @@ func defaultAnalyzers() map[string]*Analyzer {
 		cgocall.Analyzer.Name:       {Analyzer: cgocall.Analyzer, Enabled: true},
 		composite.Analyzer.Name:     {Analyzer: composite.Analyzer, Enabled: true},
 		copylock.Analyzer.Name:      {Analyzer: copylock.Analyzer, Enabled: true},
+		defers.Analyzer.Name:        {Analyzer: defers.Analyzer, Enabled: true},
+		deprecated.Analyzer.Name:    {Analyzer: deprecated.Analyzer, Enabled: true, Severity: protocol.SeverityHint, Tag: []protocol.DiagnosticTag{protocol.Deprecated}},
+		directive.Analyzer.Name:     {Analyzer: directive.Analyzer, Enabled: true},
 		errorsas.Analyzer.Name:      {Analyzer: errorsas.Analyzer, Enabled: true},
 		httpresponse.Analyzer.Name:  {Analyzer: httpresponse.Analyzer, Enabled: true},
 		ifaceassert.Analyzer.Name:   {Analyzer: ifaceassert.Analyzer, Enabled: true},
@@ -1477,9 +1576,13 @@ func defaultAnalyzers() map[string]*Analyzer {
 		unusedparams.Analyzer.Name:     {Analyzer: unusedparams.Analyzer, Enabled: false},
 		unusedwrite.Analyzer.Name:      {Analyzer: unusedwrite.Analyzer, Enabled: false},
 		useany.Analyzer.Name:           {Analyzer: useany.Analyzer, Enabled: false},
-		infertypeargs.Analyzer.Name:    {Analyzer: infertypeargs.Analyzer, Enabled: true},
-		embeddirective.Analyzer.Name:   {Analyzer: embeddirective.Analyzer, Enabled: true},
 		timeformat.Analyzer.Name:       {Analyzer: timeformat.Analyzer, Enabled: true},
+		embeddirective.Analyzer.Name: {
+			Analyzer:        embeddirective.Analyzer,
+			Enabled:         true,
+			Fix:             AddEmbedImport,
+			fixesDiagnostic: fixedByImportingEmbed,
+		},
 
 		// gofmt -s suite:
 		simplifycompositelit.Analyzer.Name: {
@@ -1637,6 +1740,7 @@ func (l *LensJSON) Write(w io.Writer) {
 type AnalyzerJSON struct {
 	Name    string
 	Doc     string
+	URL     string
 	Default bool
 }
 
