@@ -54,7 +54,9 @@ const (
 var summaryFile = "summary.txt"
 
 func init() {
-	if typeparams.Enabled {
+	if testenv.Go1Point() >= 21 {
+		summaryFile = "summary_go1.21.txt"
+	} else if testenv.Go1Point() >= 18 {
 		summaryFile = "summary_go1.18.txt"
 	}
 }
@@ -75,25 +77,17 @@ type FuzzyCompletions = map[span.Span][]Completion
 type CaseSensitiveCompletions = map[span.Span][]Completion
 type RankCompletions = map[span.Span][]Completion
 type FoldingRanges = []span.Span
-type Formats = []span.Span
-type Imports = []span.Span
 type SemanticTokens = []span.Span
 type SuggestedFixes = map[span.Span][]SuggestedFix
-type FunctionExtractions = map[span.Span]span.Span
 type MethodExtractions = map[span.Span]span.Span
 type Definitions = map[span.Span]Definition
-type Implementations = map[span.Span][]span.Span
 type Highlights = map[span.Span][]span.Span
-type References = map[span.Span][]span.Span
 type Renames = map[span.Span]string
 type PrepareRenames = map[span.Span]*source.PrepareItem
-type Symbols = map[span.URI][]*symbol
 type InlayHints = []span.Span
-type WorkspaceSymbols = map[WorkspaceSymbolsTestType]map[span.URI][]string
 type Signatures = map[span.Span]*protocol.SignatureHelp
 type Links = map[span.URI][]Link
 type AddImport = map[span.URI]string
-type Hovers = map[span.Span]string
 type SelectionRanges = []span.Span
 
 type Data struct {
@@ -111,25 +105,17 @@ type Data struct {
 	CaseSensitiveCompletions CaseSensitiveCompletions
 	RankCompletions          RankCompletions
 	FoldingRanges            FoldingRanges
-	Formats                  Formats
-	Imports                  Imports
 	SemanticTokens           SemanticTokens
 	SuggestedFixes           SuggestedFixes
-	FunctionExtractions      FunctionExtractions
 	MethodExtractions        MethodExtractions
 	Definitions              Definitions
-	Implementations          Implementations
 	Highlights               Highlights
-	References               References
 	Renames                  Renames
 	InlayHints               InlayHints
 	PrepareRenames           PrepareRenames
-	Symbols                  Symbols
-	WorkspaceSymbols         WorkspaceSymbols
 	Signatures               Signatures
 	Links                    Links
 	AddImport                AddImport
-	Hovers                   Hovers
 	SelectionRanges          SelectionRanges
 
 	fragments map[string]string
@@ -140,15 +126,15 @@ type Data struct {
 	ModfileFlagAvailable bool
 
 	mappersMu sync.Mutex
-	mappers   map[span.URI]*protocol.ColumnMapper
+	mappers   map[span.URI]*protocol.Mapper
 }
 
-// TODO(adonovan): there are multiple implementations of this (undocumented)
-// interface, each of which must implement similar semantics. For example:
-// - *runner in ../cmd/test/check.go
-// - *runner in ../source/source_test.go
-// - *runner in ../lsp_test.go
-// Can we avoid this duplication?
+// The Tests interface abstracts the LSP-based implementation of the marker
+// test operators (such as @codelens) appearing in files beneath ../testdata/.
+//
+// TODO(adonovan): reduce duplication; see https://github.com/golang/go/issues/54845.
+// There is only one implementation (*runner in ../lsp_test.go), so
+// we can abolish the interface now.
 type Tests interface {
 	CallHierarchy(*testing.T, span.Span, *CallHierarchyResult)
 	CodeLens(*testing.T, span.URI, []protocol.CodeLens)
@@ -161,25 +147,17 @@ type Tests interface {
 	CaseSensitiveCompletion(*testing.T, span.Span, Completion, CompletionItems)
 	RankCompletion(*testing.T, span.Span, Completion, CompletionItems)
 	FoldingRanges(*testing.T, span.Span)
-	Format(*testing.T, span.Span)
-	Import(*testing.T, span.Span)
 	SemanticTokens(*testing.T, span.Span)
 	SuggestedFix(*testing.T, span.Span, []SuggestedFix, int)
-	FunctionExtraction(*testing.T, span.Span, span.Span)
 	MethodExtraction(*testing.T, span.Span, span.Span)
 	Definition(*testing.T, span.Span, Definition)
-	Implementation(*testing.T, span.Span, []span.Span)
 	Highlight(*testing.T, span.Span, []span.Span)
 	InlayHints(*testing.T, span.Span)
-	References(*testing.T, span.Span, []span.Span)
 	Rename(*testing.T, span.Span, string)
 	PrepareRename(*testing.T, span.Span, *source.PrepareItem)
-	Symbols(*testing.T, span.URI, []protocol.DocumentSymbol)
-	WorkspaceSymbols(*testing.T, span.URI, string, WorkspaceSymbolsTestType)
 	SignatureHelp(*testing.T, span.Span, *protocol.SignatureHelp)
 	Link(*testing.T, span.URI, []Link)
 	AddImport(*testing.T, span.URI, string)
-	Hover(*testing.T, span.Span, string)
 	SelectionRanges(*testing.T, span.Span)
 }
 
@@ -212,19 +190,6 @@ const (
 	CompletionRank
 )
 
-type WorkspaceSymbolsTestType int
-
-const (
-	// Default runs the standard workspace symbols tests.
-	WorkspaceSymbolsDefault = WorkspaceSymbolsTestType(iota)
-
-	// Fuzzy tests workspace symbols with fuzzy matching.
-	WorkspaceSymbolsFuzzy
-
-	// CaseSensitive tests workspace symbols with case sensitive.
-	WorkspaceSymbolsCaseSensitive
-)
-
 type Completion struct {
 	CompletionItems []token.Pos
 }
@@ -249,12 +214,6 @@ type SuggestedFix struct {
 	ActionKind, Title string
 }
 
-// A symbol holds a DocumentSymbol along with its parent-child edge.
-type symbol struct {
-	pSymbol      protocol.DocumentSymbol
-	id, parentID string
-}
-
 type Golden struct {
 	Filename string
 	Archive  *txtar.Archive
@@ -286,7 +245,6 @@ func DefaultOptions(o *source.Options) {
 	o.InsertTextFormat = protocol.SnippetTextFormat
 	o.CompletionBudget = time.Minute
 	o.HierarchicalDocumentSymbolSupport = true
-	o.ExperimentalWorkspaceModule = true
 	o.SemanticTokens = true
 	o.InternalOptions.NewDiff = "both"
 }
@@ -320,26 +278,20 @@ func load(t testing.TB, mode string, dir string) *Data {
 		RankCompletions:          make(RankCompletions),
 		CaseSensitiveCompletions: make(CaseSensitiveCompletions),
 		Definitions:              make(Definitions),
-		Implementations:          make(Implementations),
 		Highlights:               make(Highlights),
-		References:               make(References),
 		Renames:                  make(Renames),
 		PrepareRenames:           make(PrepareRenames),
 		SuggestedFixes:           make(SuggestedFixes),
-		FunctionExtractions:      make(FunctionExtractions),
 		MethodExtractions:        make(MethodExtractions),
-		Symbols:                  make(Symbols),
-		WorkspaceSymbols:         make(WorkspaceSymbols),
 		Signatures:               make(Signatures),
 		Links:                    make(Links),
 		AddImport:                make(AddImport),
-		Hovers:                   make(Hovers),
 
 		dir:       dir,
 		fragments: map[string]string{},
 		golden:    map[string]*Golden{},
 		mode:      mode,
-		mappers:   map[span.URI]*protocol.ColumnMapper{},
+		mappers:   map[span.URI]*protocol.Mapper{},
 	}
 
 	if !*UpdateGolden {
@@ -466,51 +418,41 @@ func load(t testing.TB, mode string, dir string) *Data {
 
 	// Collect any data that needs to be used by subsequent tests.
 	if err := datum.Exported.Expect(map[string]interface{}{
-		"codelens":        datum.collectCodeLens,
-		"diag":            datum.collectDiagnostics,
-		"item":            datum.collectCompletionItems,
-		"complete":        datum.collectCompletions(CompletionDefault),
-		"unimported":      datum.collectCompletions(CompletionUnimported),
-		"deep":            datum.collectCompletions(CompletionDeep),
-		"fuzzy":           datum.collectCompletions(CompletionFuzzy),
-		"casesensitive":   datum.collectCompletions(CompletionCaseSensitive),
-		"rank":            datum.collectCompletions(CompletionRank),
-		"snippet":         datum.collectCompletionSnippets,
-		"fold":            datum.collectFoldingRanges,
-		"format":          datum.collectFormats,
-		"import":          datum.collectImports,
-		"semantic":        datum.collectSemanticTokens,
-		"godef":           datum.collectDefinitions,
-		"implementations": datum.collectImplementations,
-		"typdef":          datum.collectTypeDefinitions,
-		"hoverdef":        datum.collectHoverDefinitions,
-		"hover":           datum.collectHovers,
-		"highlight":       datum.collectHighlights,
-		"inlayHint":       datum.collectInlayHints,
-		"refs":            datum.collectReferences,
-		"rename":          datum.collectRenames,
-		"prepare":         datum.collectPrepareRenames,
-		"symbol":          datum.collectSymbols,
-		"signature":       datum.collectSignatures,
-		"link":            datum.collectLinks,
-		"suggestedfix":    datum.collectSuggestedFixes,
-		"extractfunc":     datum.collectFunctionExtractions,
-		"extractmethod":   datum.collectMethodExtractions,
-		"incomingcalls":   datum.collectIncomingCalls,
-		"outgoingcalls":   datum.collectOutgoingCalls,
-		"addimport":       datum.collectAddImports,
-		"selectionrange":  datum.collectSelectionRanges,
+		"codelens":       datum.collectCodeLens,
+		"diag":           datum.collectDiagnostics,
+		"item":           datum.collectCompletionItems,
+		"complete":       datum.collectCompletions(CompletionDefault),
+		"unimported":     datum.collectCompletions(CompletionUnimported),
+		"deep":           datum.collectCompletions(CompletionDeep),
+		"fuzzy":          datum.collectCompletions(CompletionFuzzy),
+		"casesensitive":  datum.collectCompletions(CompletionCaseSensitive),
+		"rank":           datum.collectCompletions(CompletionRank),
+		"snippet":        datum.collectCompletionSnippets,
+		"fold":           datum.collectFoldingRanges,
+		"semantic":       datum.collectSemanticTokens,
+		"godef":          datum.collectDefinitions,
+		"typdef":         datum.collectTypeDefinitions,
+		"hoverdef":       datum.collectHoverDefinitions,
+		"highlight":      datum.collectHighlights,
+		"inlayHint":      datum.collectInlayHints,
+		"rename":         datum.collectRenames,
+		"prepare":        datum.collectPrepareRenames,
+		"signature":      datum.collectSignatures,
+		"link":           datum.collectLinks,
+		"suggestedfix":   datum.collectSuggestedFixes,
+		"extractmethod":  datum.collectMethodExtractions,
+		"incomingcalls":  datum.collectIncomingCalls,
+		"outgoingcalls":  datum.collectOutgoingCalls,
+		"addimport":      datum.collectAddImports,
+		"selectionrange": datum.collectSelectionRanges,
 	}); err != nil {
 		t.Fatal(err)
 	}
 
 	// Collect names for the entries that require golden files.
 	if err := datum.Exported.Expect(map[string]interface{}{
-		"godef":                        datum.collectDefinitionNames,
-		"hoverdef":                     datum.collectDefinitionNames,
-		"workspacesymbol":              datum.collectWorkspaceSymbols(WorkspaceSymbolsDefault),
-		"workspacesymbolfuzzy":         datum.collectWorkspaceSymbols(WorkspaceSymbolsFuzzy),
-		"workspacesymbolcasesensitive": datum.collectWorkspaceSymbols(WorkspaceSymbolsCaseSensitive),
+		"godef":    datum.collectDefinitionNames,
+		"hoverdef": datum.collectDefinitionNames,
 	}); err != nil {
 		t.Fatal(err)
 	}
@@ -678,26 +620,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Format", func(t *testing.T) {
-		t.Helper()
-		for _, spn := range data.Formats {
-			t.Run(uriName(spn.URI()), func(t *testing.T) {
-				t.Helper()
-				tests.Format(t, spn)
-			})
-		}
-	})
-
-	t.Run("Import", func(t *testing.T) {
-		t.Helper()
-		for _, spn := range data.Imports {
-			t.Run(uriName(spn.URI()), func(t *testing.T) {
-				t.Helper()
-				tests.Import(t, spn)
-			})
-		}
-	})
-
 	t.Run("SemanticTokens", func(t *testing.T) {
 		t.Helper()
 		for _, spn := range data.SemanticTokens {
@@ -722,20 +644,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("FunctionExtraction", func(t *testing.T) {
-		t.Helper()
-		for start, end := range data.FunctionExtractions {
-			// Check if we should skip this spn if the -modfile flag is not available.
-			if shouldSkip(data, start.URI()) {
-				continue
-			}
-			t.Run(SpanName(start), func(t *testing.T) {
-				t.Helper()
-				tests.FunctionExtraction(t, start, end)
-			})
-		}
-	})
-
 	t.Run("MethodExtraction", func(t *testing.T) {
 		t.Helper()
 		for start, end := range data.MethodExtractions {
@@ -763,16 +671,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Implementation", func(t *testing.T) {
-		t.Helper()
-		for spn, m := range data.Implementations {
-			t.Run(SpanName(spn), func(t *testing.T) {
-				t.Helper()
-				tests.Implementation(t, spn, m)
-			})
-		}
-	})
-
 	t.Run("Highlight", func(t *testing.T) {
 		t.Helper()
 		for pos, locations := range data.Highlights {
@@ -783,16 +681,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Hover", func(t *testing.T) {
-		t.Helper()
-		for pos, info := range data.Hovers {
-			t.Run(SpanName(pos), func(t *testing.T) {
-				t.Helper()
-				tests.Hover(t, pos, info)
-			})
-		}
-	})
-
 	t.Run("InlayHints", func(t *testing.T) {
 		t.Helper()
 		for _, src := range data.InlayHints {
@@ -803,16 +691,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("References", func(t *testing.T) {
-		t.Helper()
-		for src, itemList := range data.References {
-			t.Run(SpanName(src), func(t *testing.T) {
-				t.Helper()
-				tests.References(t, src, itemList)
-			})
-		}
-	})
-
 	t.Run("Renames", func(t *testing.T) {
 		t.Helper()
 		for spn, newText := range data.Renames {
@@ -833,74 +711,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Symbols", func(t *testing.T) {
-		t.Helper()
-		for uri, allSymbols := range data.Symbols {
-			byParent := make(map[string][]*symbol)
-			for _, sym := range allSymbols {
-				if sym.parentID != "" {
-					byParent[sym.parentID] = append(byParent[sym.parentID], sym)
-				}
-			}
-
-			// collectChildren does a depth-first traversal of the symbol tree,
-			// computing children of child nodes before returning to their parent.
-			// This is necessary as the Children field is slice of non-pointer types,
-			// and therefore we need to be careful to mutate children first before
-			// assigning them to their parent.
-			var collectChildren func(id string) []protocol.DocumentSymbol
-			collectChildren = func(id string) []protocol.DocumentSymbol {
-				children := byParent[id]
-				// delete from byParent before recursing, to ensure that
-				// collectChildren terminates even in the presence of cycles.
-				delete(byParent, id)
-				var result []protocol.DocumentSymbol
-				for _, child := range children {
-					child.pSymbol.Children = collectChildren(child.id)
-					result = append(result, child.pSymbol)
-				}
-				return result
-			}
-
-			var topLevel []protocol.DocumentSymbol
-			for _, sym := range allSymbols {
-				if sym.parentID == "" {
-					sym.pSymbol.Children = collectChildren(sym.id)
-					topLevel = append(topLevel, sym.pSymbol)
-				}
-			}
-
-			t.Run(uriName(uri), func(t *testing.T) {
-				t.Helper()
-				tests.Symbols(t, uri, topLevel)
-			})
-		}
-	})
-
-	t.Run("WorkspaceSymbols", func(t *testing.T) {
-		t.Helper()
-
-		for _, typ := range []WorkspaceSymbolsTestType{
-			WorkspaceSymbolsDefault,
-			WorkspaceSymbolsCaseSensitive,
-			WorkspaceSymbolsFuzzy,
-		} {
-			for uri, cases := range data.WorkspaceSymbols[typ] {
-				for _, query := range cases {
-					name := query
-					if name == "" {
-						name = "EmptyQuery"
-					}
-					t.Run(name, func(t *testing.T) {
-						t.Helper()
-						tests.WorkspaceSymbols(t, uri, query, typ)
-					})
-				}
-			}
-		}
-
-	})
-
 	t.Run("SignatureHelp", func(t *testing.T) {
 		t.Helper()
 		for spn, expectedSignature := range data.Signatures {
@@ -1005,15 +815,6 @@ func checkData(t *testing.T, data *Data) {
 		return count
 	}
 
-	countWorkspaceSymbols := func(c map[WorkspaceSymbolsTestType]map[span.URI][]string) (count int) {
-		for _, typs := range c {
-			for _, queries := range typs {
-				count += len(queries)
-			}
-		}
-		return count
-	}
-
 	fmt.Fprintf(buf, "CallHierarchyCount = %v\n", len(data.CallHierarchy))
 	fmt.Fprintf(buf, "CodeLensCount = %v\n", countCodeLens(data.CodeLens))
 	fmt.Fprintf(buf, "CompletionsCount = %v\n", countCompletions(data.Completions))
@@ -1025,24 +826,17 @@ func checkData(t *testing.T, data *Data) {
 	fmt.Fprintf(buf, "CaseSensitiveCompletionsCount = %v\n", countCompletions(data.CaseSensitiveCompletions))
 	fmt.Fprintf(buf, "DiagnosticsCount = %v\n", diagnosticsCount)
 	fmt.Fprintf(buf, "FoldingRangesCount = %v\n", len(data.FoldingRanges))
-	fmt.Fprintf(buf, "FormatCount = %v\n", len(data.Formats))
-	fmt.Fprintf(buf, "ImportCount = %v\n", len(data.Imports))
 	fmt.Fprintf(buf, "SemanticTokenCount = %v\n", len(data.SemanticTokens))
 	fmt.Fprintf(buf, "SuggestedFixCount = %v\n", len(data.SuggestedFixes))
-	fmt.Fprintf(buf, "FunctionExtractionCount = %v\n", len(data.FunctionExtractions))
 	fmt.Fprintf(buf, "MethodExtractionCount = %v\n", len(data.MethodExtractions))
 	fmt.Fprintf(buf, "DefinitionsCount = %v\n", definitionCount)
 	fmt.Fprintf(buf, "TypeDefinitionsCount = %v\n", typeDefinitionCount)
 	fmt.Fprintf(buf, "HighlightsCount = %v\n", len(data.Highlights))
 	fmt.Fprintf(buf, "InlayHintsCount = %v\n", len(data.InlayHints))
-	fmt.Fprintf(buf, "ReferencesCount = %v\n", len(data.References))
 	fmt.Fprintf(buf, "RenamesCount = %v\n", len(data.Renames))
 	fmt.Fprintf(buf, "PrepareRenamesCount = %v\n", len(data.PrepareRenames))
-	fmt.Fprintf(buf, "SymbolsCount = %v\n", len(data.Symbols))
-	fmt.Fprintf(buf, "WorkspaceSymbolsCount = %v\n", countWorkspaceSymbols(data.WorkspaceSymbols))
 	fmt.Fprintf(buf, "SignaturesCount = %v\n", len(data.Signatures))
 	fmt.Fprintf(buf, "LinksCount = %v\n", linksCount)
-	fmt.Fprintf(buf, "ImplementationsCount = %v\n", len(data.Implementations))
 	fmt.Fprintf(buf, "SelectionRangesCount = %v\n", len(data.SelectionRanges))
 
 	want := string(data.Golden(t, "summary", summaryFile, func() ([]byte, error) {
@@ -1057,7 +851,7 @@ func checkData(t *testing.T, data *Data) {
 	}
 }
 
-func (data *Data) Mapper(uri span.URI) (*protocol.ColumnMapper, error) {
+func (data *Data) Mapper(uri span.URI) (*protocol.Mapper, error) {
 	data.mappersMu.Lock()
 	defer data.mappersMu.Unlock()
 
@@ -1066,7 +860,7 @@ func (data *Data) Mapper(uri span.URI) (*protocol.ColumnMapper, error) {
 		if err != nil {
 			return nil, err
 		}
-		data.mappers[uri] = protocol.NewColumnMapper(uri, content)
+		data.mappers[uri] = protocol.NewMapper(uri, content)
 	}
 	return data.mappers[uri], nil
 }
@@ -1127,7 +921,7 @@ func (data *Data) Golden(t *testing.T, tag, target string, update func() ([]byte
 func (data *Data) collectCodeLens(spn span.Span, title, cmd string) {
 	data.CodeLens[spn.URI()] = append(data.CodeLens[spn.URI()], protocol.CodeLens{
 		Range: data.mustRange(spn),
-		Command: protocol.Command{
+		Command: &protocol.Command{
 			Title:   title,
 			Command: cmd,
 		},
@@ -1206,14 +1000,6 @@ func (data *Data) collectFoldingRanges(spn span.Span) {
 	data.FoldingRanges = append(data.FoldingRanges, spn)
 }
 
-func (data *Data) collectFormats(spn span.Span) {
-	data.Formats = append(data.Formats, spn)
-}
-
-func (data *Data) collectImports(spn span.Span) {
-	data.Imports = append(data.Imports, spn)
-}
-
 func (data *Data) collectAddImports(spn span.Span, imp string) {
 	data.AddImport[spn.URI()] = imp
 }
@@ -1226,12 +1012,6 @@ func (data *Data) collectSuggestedFixes(spn span.Span, actionKind, fix string) {
 	data.SuggestedFixes[spn] = append(data.SuggestedFixes[spn], SuggestedFix{actionKind, fix})
 }
 
-func (data *Data) collectFunctionExtractions(start span.Span, end span.Span) {
-	if _, ok := data.FunctionExtractions[start]; !ok {
-		data.FunctionExtractions[start] = end
-	}
-}
-
 func (data *Data) collectMethodExtractions(start span.Span, end span.Span) {
 	if _, ok := data.MethodExtractions[start]; !ok {
 		data.MethodExtractions[start] = end
@@ -1249,10 +1029,6 @@ func (data *Data) collectSelectionRanges(spn span.Span) {
 	data.SelectionRanges = append(data.SelectionRanges, spn)
 }
 
-func (data *Data) collectImplementations(src span.Span, targets []span.Span) {
-	data.Implementations[src] = targets
-}
-
 func (data *Data) collectIncomingCalls(src span.Span, calls []span.Span) {
 	for _, call := range calls {
 		rng := data.mustRange(call)
@@ -1295,10 +1071,6 @@ func (data *Data) collectHoverDefinitions(src, target span.Span) {
 	}
 }
 
-func (data *Data) collectHovers(src span.Span, expected string) {
-	data.Hovers[src] = expected
-}
-
 func (data *Data) collectTypeDefinitions(src, target span.Span) {
 	data.Definitions[src] = Definition{
 		Src:    src,
@@ -1322,10 +1094,6 @@ func (data *Data) collectInlayHints(src span.Span) {
 	data.InlayHints = append(data.InlayHints, src)
 }
 
-func (data *Data) collectReferences(src span.Span, expected []span.Span) {
-	data.References[src] = expected
-}
-
 func (data *Data) collectRenames(src span.Span, newText string) {
 	data.Renames[src] = newText
 }
@@ -1337,44 +1105,16 @@ func (data *Data) collectPrepareRenames(src, spn span.Span, placeholder string)
 	}
 }
 
-// collectSymbols is responsible for collecting @symbol annotations.
-func (data *Data) collectSymbols(name string, selectionRng span.Span, kind, detail, id, parentID string) {
-	// We don't set 'Range' here as it is difficult (impossible?) to express
-	// multi-line ranges in the packagestest framework.
-	uri := selectionRng.URI()
-	data.Symbols[uri] = append(data.Symbols[uri], &symbol{
-		pSymbol: protocol.DocumentSymbol{
-			Name:           name,
-			Kind:           protocol.ParseSymbolKind(kind),
-			SelectionRange: data.mustRange(selectionRng),
-			Detail:         detail,
-		},
-		id:       id,
-		parentID: parentID,
-	})
-}
-
 // mustRange converts spn into a protocol.Range, panicking on any error.
 func (data *Data) mustRange(spn span.Span) protocol.Range {
 	m, err := data.Mapper(spn.URI())
-	rng, err := m.Range(spn)
+	rng, err := m.SpanRange(spn)
 	if err != nil {
 		panic(fmt.Sprintf("converting span %s to range: %v", spn, err))
 	}
 	return rng
 }
 
-func (data *Data) collectWorkspaceSymbols(typ WorkspaceSymbolsTestType) func(*expect.Note, string) {
-	return func(note *expect.Note, query string) {
-		if data.WorkspaceSymbols[typ] == nil {
-			data.WorkspaceSymbols[typ] = make(map[span.URI][]string)
-		}
-		pos := safetoken.StartPosition(data.Exported.ExpectFileSet, note.Pos)
-		uri := span.URIFromPath(pos.Filename)
-		data.WorkspaceSymbols[typ][uri] = append(data.WorkspaceSymbols[typ][uri], query)
-	}
-}
-
 func (data *Data) collectSignatures(spn span.Span, signature string, activeParam int64) {
 	data.Signatures[spn] = &protocol.SignatureHelp{
 		Signatures: []protocol.SignatureInformation{
