@@ -16,10 +16,12 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+
 	"golang.org/x/tools/gopls/internal/govulncheck"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	. "golang.org/x/tools/gopls/internal/lsp/regtest"
+	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/gopls/internal/vulncheck/vulntest"
@@ -225,12 +227,10 @@ func main() {
 			Arguments: lens.Command.Arguments,
 		}, &result)
 
-		env.Await(
-			OnceMet(
-				CompletedProgress(result.Token, nil),
-				ShownMessage("Found GOSTDLIB"),
-				EmptyOrNoDiagnostics("go.mod"),
-			),
+		env.OnceMet(
+			CompletedProgress(result.Token, nil),
+			ShownMessage("Found GOSTDLIB"),
+			NoDiagnostics(ForFile("go.mod")),
 		)
 		testFetchVulncheckResult(t, env, map[string]fetchVulncheckResult{
 			"go.mod": {IDs: []string{"GOSTDLIB"}, Mode: govulncheck.ModeGovulncheck}})
@@ -276,7 +276,7 @@ func main() {
 	).Run(t, files, func(t *testing.T, env *Env) {
 		env.OpenFile("go.mod")
 		env.AfterChange(
-			EmptyOrNoDiagnostics("go.mod"),
+			NoDiagnostics(ForFile("go.mod")),
 			// we don't publish diagnostics for standard library vulnerability yet.
 		)
 		testFetchVulncheckResult(t, env, map[string]fetchVulncheckResult{
@@ -487,7 +487,7 @@ func TestRunVulncheckPackageDiagnostics(t *testing.T) {
 
 		gotDiagnostics := &protocol.PublishDiagnosticsParams{}
 		env.AfterChange(
-			env.DiagnosticAtRegexp("go.mod", `golang.org/amod`),
+			Diagnostics(env.AtRegexp("go.mod", `golang.org/amod`)),
 			ReadDiagnostics("go.mod", gotDiagnostics),
 		)
 
@@ -504,17 +504,18 @@ func TestRunVulncheckPackageDiagnostics(t *testing.T) {
 					{
 						msg:      "golang.org/amod has known vulnerabilities GO-2022-01, GO-2022-03.",
 						severity: protocol.SeverityInformation,
+						source:   string(source.Vulncheck),
 						codeActions: []string{
-							"Upgrade to latest",
-							"Upgrade to v1.0.6",
 							"Run govulncheck to verify",
+							"Upgrade to v1.0.6",
+							"Upgrade to latest",
 						},
 					},
 				},
 				codeActions: []string{
-					"Upgrade to latest",
-					"Upgrade to v1.0.6",
 					"Run govulncheck to verify",
+					"Upgrade to v1.0.6",
+					"Upgrade to latest",
 				},
 				hover: []string{"GO-2022-01", "Fixed in v1.0.4.", "GO-2022-03"},
 			},
@@ -523,6 +524,7 @@ func TestRunVulncheckPackageDiagnostics(t *testing.T) {
 					{
 						msg:      "golang.org/bmod has a vulnerability GO-2022-02.",
 						severity: protocol.SeverityInformation,
+						source:   string(source.Vulncheck),
 						codeActions: []string{
 							"Run govulncheck to verify",
 						},
@@ -586,17 +588,13 @@ func TestRunVulncheckPackageDiagnostics(t *testing.T) {
 					var result command.RunVulncheckResult
 					env.ExecuteCodeLensCommand("go.mod", command.RunGovulncheck, &result)
 					gotDiagnostics := &protocol.PublishDiagnosticsParams{}
-					env.Await(
-						OnceMet(
-							CompletedProgress(result.Token, nil),
-							ShownMessage("Found"),
-						),
+					env.OnceMet(
+						CompletedProgress(result.Token, nil),
+						ShownMessage("Found"),
 					)
-					env.Await(
-						OnceMet(
-							env.DiagnosticAtRegexp("go.mod", "golang.org/bmod"),
-							ReadDiagnostics("go.mod", gotDiagnostics),
-						),
+					env.OnceMet(
+						Diagnostics(env.AtRegexp("go.mod", "golang.org/bmod")),
+						ReadDiagnostics("go.mod", gotDiagnostics),
 					)
 					// We expect only one diagnostic for GO-2022-02.
 					count := 0
@@ -636,27 +634,23 @@ func TestRunVulncheckWarning(t *testing.T) {
 		var result command.RunVulncheckResult
 		env.ExecuteCodeLensCommand("go.mod", command.RunGovulncheck, &result)
 		gotDiagnostics := &protocol.PublishDiagnosticsParams{}
-		env.Await(
-			OnceMet(
-				CompletedProgress(result.Token, nil),
-				ShownMessage("Found"),
-			),
+		env.OnceMet(
+			CompletedProgress(result.Token, nil),
+			ShownMessage("Found"),
 		)
 		// Vulncheck diagnostics asynchronous to the vulncheck command.
-		env.Await(
-			OnceMet(
-				env.DiagnosticAtRegexp("go.mod", `golang.org/amod`),
-				ReadDiagnostics("go.mod", gotDiagnostics),
-			),
+		env.OnceMet(
+			Diagnostics(env.AtRegexp("go.mod", `golang.org/amod`)),
+			ReadDiagnostics("go.mod", gotDiagnostics),
 		)
 
 		testFetchVulncheckResult(t, env, map[string]fetchVulncheckResult{
 			"go.mod": {IDs: []string{"GO-2022-01", "GO-2022-02", "GO-2022-03"}, Mode: govulncheck.ModeGovulncheck},
 		})
 		env.OpenFile("x/x.go")
-		lineX := env.RegexpSearch("x/x.go", `c\.C1\(\)\.Vuln1\(\)`)
+		lineX := env.RegexpSearch("x/x.go", `c\.C1\(\)\.Vuln1\(\)`).Range.Start
 		env.OpenFile("y/y.go")
-		lineY := env.RegexpSearch("y/y.go", `c\.C2\(\)\(\)`)
+		lineY := env.RegexpSearch("y/y.go", `c\.C2\(\)\(\)`).Range.Start
 		wantDiagnostics := map[string]vulnDiagExpectation{
 			"golang.org/amod": {
 				applyAction: "Upgrade to v1.0.6",
@@ -664,9 +658,10 @@ func TestRunVulncheckWarning(t *testing.T) {
 					{
 						msg:      "golang.org/amod has a vulnerability used in the code: GO-2022-01.",
 						severity: protocol.SeverityWarning,
+						source:   string(source.Govulncheck),
 						codeActions: []string{
-							"Upgrade to latest",
 							"Upgrade to v1.0.4",
+							"Upgrade to latest",
 							"Reset govulncheck result",
 						},
 						relatedInfo: []vulnRelatedInfo{
@@ -677,20 +672,17 @@ func TestRunVulncheckWarning(t *testing.T) {
 					{
 						msg:      "golang.org/amod has a vulnerability GO-2022-03 that is not used in the code.",
 						severity: protocol.SeverityInformation,
+						source:   string(source.Govulncheck),
 						codeActions: []string{
-							"Upgrade to latest",
 							"Upgrade to v1.0.6",
+							"Upgrade to latest",
 							"Reset govulncheck result",
 						},
-						relatedInfo: []vulnRelatedInfo{
-							{"x.go", uint32(lineX.Line), "[GO-2022-01]"}, // avuln.VulnData.Vuln1
-							{"x.go", uint32(lineX.Line), "[GO-2022-01]"}, // avuln.VulnData.Vuln2
-						},
 					},
 				},
 				codeActions: []string{
-					"Upgrade to latest",
 					"Upgrade to v1.0.6",
+					"Upgrade to latest",
 					"Reset govulncheck result",
 				},
 				hover: []string{"GO-2022-01", "Fixed in v1.0.4.", "GO-2022-03"},
@@ -700,6 +692,7 @@ func TestRunVulncheckWarning(t *testing.T) {
 					{
 						msg:      "golang.org/bmod has a vulnerability used in the code: GO-2022-02.",
 						severity: protocol.SeverityWarning,
+						source:   string(source.Govulncheck),
 						codeActions: []string{
 							"Reset govulncheck result", // no fix, but we should give an option to reset.
 						},
@@ -806,19 +799,15 @@ func TestGovulncheckInfo(t *testing.T) {
 		var result command.RunVulncheckResult
 		env.ExecuteCodeLensCommand("go.mod", command.RunGovulncheck, &result)
 		gotDiagnostics := &protocol.PublishDiagnosticsParams{}
-		env.Await(
-			OnceMet(
-				CompletedProgress(result.Token, nil),
-				ShownMessage("No vulnerabilities found"), // only count affecting vulnerabilities.
-			),
+		env.OnceMet(
+			CompletedProgress(result.Token, nil),
+			ShownMessage("No vulnerabilities found"), // only count affecting vulnerabilities.
 		)
 
 		// Vulncheck diagnostics asynchronous to the vulncheck command.
-		env.Await(
-			OnceMet(
-				env.DiagnosticAtRegexp("go.mod", "golang.org/bmod"),
-				ReadDiagnostics("go.mod", gotDiagnostics),
-			),
+		env.OnceMet(
+			Diagnostics(env.AtRegexp("go.mod", "golang.org/bmod")),
+			ReadDiagnostics("go.mod", gotDiagnostics),
 		)
 
 		testFetchVulncheckResult(t, env, map[string]fetchVulncheckResult{"go.mod": {IDs: []string{"GO-2022-02"}, Mode: govulncheck.ModeGovulncheck}})
@@ -831,6 +820,7 @@ func TestGovulncheckInfo(t *testing.T) {
 					{
 						msg:      "golang.org/bmod has a vulnerability GO-2022-02 that is not used in the code.",
 						severity: protocol.SeverityInformation,
+						source:   string(source.Govulncheck),
 						codeActions: []string{
 							"Reset govulncheck result",
 						},
@@ -868,7 +858,7 @@ func TestGovulncheckInfo(t *testing.T) {
 		}
 		env.ApplyCodeAction(reset)
 
-		env.Await(EmptyOrNoDiagnostics("go.mod"))
+		env.Await(NoDiagnostics(ForFile("go.mod")))
 	})
 }
 
@@ -876,14 +866,14 @@ func TestGovulncheckInfo(t *testing.T) {
 // and runs checks if diagnostics and code actions associated with the line match expectation.
 func testVulnDiagnostics(t *testing.T, env *Env, pattern string, want vulnDiagExpectation, got *protocol.PublishDiagnosticsParams) []protocol.Diagnostic {
 	t.Helper()
-	pos := env.RegexpSearch("go.mod", pattern)
+	loc := env.RegexpSearch("go.mod", pattern)
 	var modPathDiagnostics []protocol.Diagnostic
 	for _, w := range want.diagnostics {
-		// Find the diagnostics at pos.
+		// Find the diagnostics at loc.start.
 		var diag *protocol.Diagnostic
 		for _, g := range got.Diagnostics {
 			g := g
-			if g.Range.Start == pos.ToProtocolPosition() && w.msg == g.Message {
+			if g.Range.Start == loc.Range.Start && w.msg == g.Message {
 				modPathDiagnostics = append(modPathDiagnostics, g)
 				diag = &g
 				break
@@ -893,8 +883,8 @@ func testVulnDiagnostics(t *testing.T, env *Env, pattern string, want vulnDiagEx
 			t.Errorf("no diagnostic at %q matching %q found\n", pattern, w.msg)
 			continue
 		}
-		if diag.Severity != w.severity {
-			t.Errorf("incorrect severity for %q, want %s got %s\n", w.msg, w.severity, diag.Severity)
+		if diag.Severity != w.severity || diag.Source != w.source {
+			t.Errorf("incorrect (severity, source) for %q, want (%s, %s) got (%s, %s)\n", w.msg, w.severity, w.source, diag.Severity, diag.Source)
 		}
 		sort.Slice(w.relatedInfo, func(i, j int) bool { return w.relatedInfo[i].less(w.relatedInfo[j]) })
 		if got, want := summarizeRelatedInfo(diag.RelatedInformation), w.relatedInfo; !cmp.Equal(got, want) {
@@ -909,7 +899,7 @@ func testVulnDiagnostics(t *testing.T, env *Env, pattern string, want vulnDiagEx
 	}
 	// Check that useful info is supplemented as hover.
 	if len(want.hover) > 0 {
-		hover, _ := env.Hover("go.mod", pos)
+		hover, _ := env.Hover(loc)
 		for _, part := range want.hover {
 			if !strings.Contains(hover.Value, part) {
 				t.Errorf("hover contents for %q do not match, want %v, got %v\n", pattern, strings.Join(want.hover, ","), hover.Value)
@@ -951,6 +941,8 @@ type vulnDiag struct {
 	// relatedInfo is related info message prefixed by the file base.
 	// See summarizeRelatedInfo.
 	relatedInfo []vulnRelatedInfo
+	// diagnostic source.
+	source string
 }
 
 func (i vulnRelatedInfo) less(j vulnRelatedInfo) bool {
