@@ -14,7 +14,6 @@ import (
 	"fmt"
 	"log"
 	"os"
-	"regexp"
 	"sort"
 	"strings"
 	"sync"
@@ -70,8 +69,12 @@ func init() {
 			return err
 		}
 		logf("Loaded %d packages and their dependencies", len(pkgs))
+		cache, err := govulncheck.DefaultCache()
+		if err != nil {
+			return err
+		}
 		cli, err := client.NewClient(findGOVULNDB(cfg.Env), client.Options{
-			HTTPCache: govulncheck.DefaultCache(),
+			HTTPCache: cache,
 		})
 		if err != nil {
 			return err
@@ -97,49 +100,6 @@ func init() {
 	}
 }
 
-var (
-	// Regexp for matching go tags. The groups are:
-	// 1  the major.minor version
-	// 2  the patch version, or empty if none
-	// 3  the entire prerelease, if present
-	// 4  the prerelease type ("beta" or "rc")
-	// 5  the prerelease number
-	tagRegexp = regexp.MustCompile(`^go(\d+\.\d+)(\.\d+|)((beta|rc|-pre)(\d+))?$`)
-)
-
-// This is a modified copy of pkgsite/internal/stdlib:VersionForTag.
-func GoTagToSemver(tag string) string {
-	if tag == "" {
-		return ""
-	}
-
-	tag = strings.Fields(tag)[0]
-	// Special cases for go1.
-	if tag == "go1" {
-		return "v1.0.0"
-	}
-	if tag == "go1.0" {
-		return ""
-	}
-	m := tagRegexp.FindStringSubmatch(tag)
-	if m == nil {
-		return ""
-	}
-	version := "v" + m[1]
-	if m[2] != "" {
-		version += m[2]
-	} else {
-		version += ".0"
-	}
-	if m[3] != "" {
-		if !strings.HasPrefix(m[4], "-") {
-			version += "-"
-		}
-		version += m[4] + "." + m[5]
-	}
-	return version
-}
-
 // semverToGoTag returns the Go standard library repository tag corresponding
 // to semver, a version string without the initial "v".
 // Go tags differ from standard semantic versions in a few ways,
@@ -232,9 +192,13 @@ func vulnerablePackages(ctx context.Context, snapshot source.Snapshot, modfile s
 	}
 
 	// Request vuln entries from remote service.
+	fsCache, err := govulncheck.DefaultCache()
+	if err != nil {
+		return nil, err
+	}
 	cli, err := client.NewClient(
 		findGOVULNDB(snapshot.View().Options().EnvSlice()),
-		client.Options{HTTPCache: govulncheck.NewInMemoryCache(govulncheck.DefaultCache())})
+		client.Options{HTTPCache: govulncheck.NewInMemoryCache(fsCache)})
 	if err != nil {
 		return nil, err
 	}
